#include "contourwidget.h"

ContourWidget::ContourWidget( const Image& image )
	:	ToolWidget( new QTabWidget(), image, true )
{
	if( m_image.GetChanNumber() > 1 )
	{
		ConvertToGrayscale( &m_image );
	}
	
	m_gradient = new Gradient();
	m_seuil1 = 20;
	m_seuil2 = 100;
	m_seuil = 100;
	m_seuil = 40;
	
	m_type = 0;
	
	m_mask_gradient_widget = new MaskGradientWidget();
	connect( m_mask_gradient_widget, SIGNAL( maskChanged( Gradient::GradientMaskType ) ), this, SLOT( convolution( Gradient::GradientMaskType ) ) );
	
	m_seuillage_widget = new SeuillageWidget();
	connect( m_seuillage_widget, SIGNAL( seuilHysteresisChanged( unsigned char, unsigned char ) ), this, SLOT( seuillageHysteresis( unsigned char, unsigned char ) ) );
	connect( m_seuillage_widget, SIGNAL( seuilGlobalChanged( unsigned char ) ), this, SLOT( seuillageGlobal( unsigned char ) ) );
	connect( m_seuillage_widget, SIGNAL( seuilLocalChanged( int ) ), this, SLOT( seuillageLocal( int ) ) );
	
	m_refine_widget = new QWidget();
	m_link_widget = new QWidget();
	
	static_cast<QTabWidget*>(m_widget)->addTab( m_mask_gradient_widget, tr("Convolution") );
	static_cast<QTabWidget*>(m_widget)->addTab( m_seuillage_widget, tr("Seuillage") );
	static_cast<QTabWidget*>(m_widget)->addTab( m_refine_widget, tr("Affinage") );
	static_cast<QTabWidget*>(m_widget)->addTab( m_link_widget, tr("Fermeture") );
	
	connect( static_cast<QTabWidget*>(m_widget), SIGNAL( currentChanged( int ) ), this, SLOT( currentTabChanged( int ) ) );
}
ContourWidget::~ContourWidget()
{
}

void ContourWidget::init()
{
	m_mask_gradient_widget->activate();
}

void ContourWidget::convolution( Gradient::GradientMaskType mask )
{
	start();
	m_maskType = mask;
	m_gradient->ComputeFromImage( &m_image, mask );
	previewImage( *m_gradient->GetResult() );
	stop();
}

void ContourWidget::seuillageHysteresis( unsigned char seuil1, unsigned char seuil2 )
{
	start();
	m_type = 0;
	m_seuil1 = seuil1;
	m_seuil2 = seuil2;
	m_gradient->ThresholdHysteresis( seuil1, seuil2 );
	previewImage( *m_gradient->GetResult() );
	stop();
}
void ContourWidget::seuillageGlobal( unsigned char seuil )
{
	start();
	m_type = 1;
	m_seuil = seuil;
	m_gradient->ThresholdGlobal( seuil );
	previewImage( *m_gradient->GetResult() );
	stop();
}
void ContourWidget::seuillageLocal( int size )
{
	start();
	m_type = 2;
	m_size = size;
	m_gradient->ThresholdLocal( size );
	previewImage( *m_gradient->GetResult() );
	stop();
}

void ContourWidget::refine()
{
	start();
	if( m_type == 0 )
		m_gradient->ThresholdHysteresis( m_seuil1, m_seuil2 );
	else if( m_type == 1 )
		m_gradient->ThresholdGlobal( m_seuil );
	else if( m_type == 2 )
		m_gradient->ThresholdLocal( m_size );
	m_gradient->Refine();
	previewImage( *m_gradient->GetResult() );
	stop();
}
void ContourWidget::link()
{
	start();
	if( m_type == 0 )
		m_gradient->ThresholdHysteresis( m_seuil1, m_seuil2 );
	else if( m_type == 1 )
		m_gradient->ThresholdGlobal( m_seuil );
	else if( m_type == 2 )
		m_gradient->ThresholdLocal( m_size );
	m_gradient->Refine();
	m_gradient->Link();
	previewImage( *m_gradient->GetResult() );
	stop();
}


void ContourWidget::currentTabChanged( int index )
{
	if( index < 2 )
		static_cast<ParamWidget*>(static_cast<QTabWidget*>(m_widget)->currentWidget())->activate();
	else
	{
		if( index == 2 )
		{
			refine();
		}
		else
		{
			link();
		}
	}
}
