#include "filterwidget.h"

FilterWidget::FilterWidget( const Image& image )
	:	ToolWidget( new QTabWidget(), image )
{
	m_mask_widget = new MaskWidget();
	connect( m_mask_widget, SIGNAL( maskChanged( int* ) ), this, SLOT( convolution( int* ) ) );
	m_morpho_widget = new MorphoWidget( qMin( image.GetWidth(), image.GetHeight() ) / 2 - 1);
	connect( m_morpho_widget, SIGNAL( changed( MorphoType, int ) ), this, SLOT( morpho( MorphoType, int ) ) );
	m_nagao_widget = new NagaoWidget();
	connect( m_nagao_widget, SIGNAL( changed() ), this, SLOT( nagao() ) );
	static_cast<QTabWidget*>(m_widget)->addTab( m_mask_widget, tr("Convolution") );
	static_cast<QTabWidget*>(m_widget)->addTab( m_morpho_widget, tr("Morphologique") );
	static_cast<QTabWidget*>(m_widget)->addTab( m_nagao_widget, tr("Nagao") );

	connect( static_cast<QTabWidget*>(m_widget), SIGNAL( currentChanged( int ) ), this, SLOT( currentTabChanged( int ) ) );
}
FilterWidget::~FilterWidget()
{
}

void FilterWidget::init()
{
	m_mask_widget->activate();	
}

void FilterWidget::convolution( int* mask )
{
	start();
	previewImage( Filtre::convolution( m_image, mask, 1 ) );
	stop();
}
void FilterWidget::morpho( MorphoType type, int size )
{
	start();
	if( type == Median )
	{
		previewImage( Filtre::fast_median( m_image, size ) );
	}
	else if( type == Minimum )
	{
		previewImage( Filtre::fast_minimum( m_image, size ) );
	}
	else if( type == Maximum )
	{
		previewImage( Filtre::fast_maximum( m_image, size ) );
	}
	stop();
}
void FilterWidget::nagao( )
{
	start();
	previewImage( Filtre::nagao( m_image ) );
	stop();
}

void FilterWidget::currentTabChanged( int )
{
	static_cast<ParamWidget*>(static_cast<QTabWidget*>(m_widget)->currentWidget())->activate();
}
