#include "fourierwidget.h"

FourierWidget::FourierWidget( const Image& image )
	:	ToolWidget( new QTabWidget(), image )
{
	if( image.GetWidth() != image.GetHeight() || (image.GetWidth() & (image.GetWidth()-1)) != 0 )
	{
		QMessageBox::warning( NULL, tr("Erreur"), tr("Les dimensions de l'image ne sont pas une puissance de 2.") );
		m_fourier = NULL;
	}
	else
	{
		m_fourier = new FourierMT( image );
		m_fourier->Save();

		m_ideal_filter_slider = new QSlider( );
		m_ideal_filter_slider->setRange( 1, image.GetWidth()/2-1 );
		m_ideal_filter_slider->setValue( 60 );
		m_ideal_filter_slider->setOrientation( Qt::Horizontal );
		connect( m_ideal_filter_slider, SIGNAL( valueChanged( int ) ), this, SLOT( idealFilter( int ) ) );
		m_gaussian_filter_slider = new QSlider( );
		m_gaussian_filter_slider->setRange( 0, 255 );
		m_gaussian_filter_slider->setValue( 10 );
		m_gaussian_filter_slider->setOrientation( Qt::Horizontal );
		connect( m_gaussian_filter_slider, SIGNAL( valueChanged( int ) ), this, SLOT( gaussianFilter( int ) ) );
		m_ideal_filter_lcd = new QLCDNumber( 3 );
		m_gaussian_filter_lcd = new QLCDNumber( 3 );
		m_ideal_filter_lcd->display( m_ideal_filter_slider->value() ) ;
		m_gaussian_filter_lcd->display( m_gaussian_filter_slider->value() ) ;
		
		m_ideal_filter_label = new QLabel( tr( "Rayon :" ) );
		m_gaussian_filter_label = new QLabel( tr( "Sigma :" ) );
		
		m_ideal_filter_layout = new QHBoxLayout();
		m_ideal_filter_layout->addWidget( m_ideal_filter_label );
		m_ideal_filter_layout->addWidget( m_ideal_filter_slider );
		m_ideal_filter_layout->addWidget( m_ideal_filter_lcd );
		m_gaussian_filter_layout = new QHBoxLayout();
		m_gaussian_filter_layout->addWidget( m_gaussian_filter_label );
		m_gaussian_filter_layout->addWidget( m_gaussian_filter_slider );
		m_gaussian_filter_layout->addWidget( m_gaussian_filter_lcd );
		
		m_ideal_filter_widget = new QWidget();
		m_gaussian_filter_widget = new QWidget();
		m_module_widget = new QWidget();
		m_ideal_filter_widget->setLayout( m_ideal_filter_layout );
		m_gaussian_filter_widget->setLayout( m_gaussian_filter_layout );

		static_cast<QTabWidget*>(m_widget)->addTab( m_ideal_filter_widget, tr("Filtre ideal") );
		static_cast<QTabWidget*>(m_widget)->addTab( m_gaussian_filter_widget, tr("Filtre gaussien") );
		static_cast<QTabWidget*>(m_widget)->addTab( m_module_widget, tr("Module") );

		connect( static_cast<QTabWidget*>(m_widget), SIGNAL( currentChanged( int ) ), this, SLOT( currentTabChanged( int ) ) );
	}
}
FourierWidget::~FourierWidget()
{
	if( m_fourier ) delete m_fourier;
}

void FourierWidget::init()
{
	if( m_fourier ) currentTabChanged( 0 );
}
int FourierWidget::exec()
{
	if( !m_fourier ) return QDialog::Rejected;
	else return QDialog::exec();
}

void FourierWidget::updatePreview()
{
	previewImage( m_fourier->FastTransformInverse() );
	stop();
}

void FourierWidget::idealFilter( int radius )
{
	m_ideal_filter_lcd->display( m_ideal_filter_slider->value() ) ;
	m_fourier->Load();
	start();
	m_fourier->IdealFilter( radius );
	updatePreview();
}
void FourierWidget::gaussianFilter( int sigma )
{
	m_gaussian_filter_lcd->display( m_gaussian_filter_slider->value() ) ;
	m_fourier->Load();
	start();
	m_fourier->GaussianFilter( sigma / 255.0 );
	updatePreview();
}
void FourierWidget::showModule()
{
	m_fourier->Load();
	start();
	previewImage( m_fourier->GetResult() );
	stop();
}

void FourierWidget::currentTabChanged( int index )
{
	switch( index )
	{
	case 0:
		idealFilter( m_ideal_filter_slider->value() );
		break;
	case 1:
		gaussianFilter( m_gaussian_filter_slider->value() );
		break;
	case 2:
		showModule();
		break;
	}
}
