#include "houghwidget.h"

HoughWidget::HoughWidget( const Image& image )
	:	ToolWidget( new QTabWidget(), image )
{
	bool is_binary = true;
	for( unsigned int i = 0 ; i < image.GetWidth() * image.GetHeight() ; i++ )
	{
		unsigned char* ptr_im = image.GetPixel( i );
		for( unsigned int k = 0 ; k < image.GetChanNumber() ; k++ )
		{
			if( ptr_im[k] != 0 && ptr_im[k] != 255 )
				is_binary = false;
		}
	}
	
	if( !is_binary )
	{
		QMessageBox::warning( NULL, tr("Erreur"), tr("L'image doit etre binaire.") );
		m_hough = NULL;
	}
	else
	{
		m_hough = new Hough( image );
		
		m_line_slider = new QSlider( );
		m_line_slider->setRange( 1, qMin( image.GetWidth(), image.GetHeight() ) / 2 );
		m_line_slider->setValue( 1 );
		m_line_slider->setOrientation( Qt::Horizontal );
		connect( m_line_slider, SIGNAL( valueChanged( int ) ), this, SLOT( line( int ) ) );
		m_circle_slider = new QSlider( );
		m_circle_slider->setRange( 1, qMin( image.GetWidth(), image.GetHeight() ) / 2 );
		m_circle_slider->setValue( 1 );
		m_circle_slider->setOrientation( Qt::Horizontal );
		connect( m_circle_slider, SIGNAL( valueChanged( int ) ), this, SLOT( circle( int ) ) );
		
		m_line_lcd = new QLCDNumber( 3 );
		m_circle_lcd = new QLCDNumber( 3 );
		m_line_lcd->display( m_line_slider->value() ) ;
		m_circle_lcd->display( m_circle_slider->value() ) ;
		
		m_line_label = new QLabel( tr( "Nombre de lignes :" ) );
		m_circle_label = new QLabel( tr( "Nombre de cercles :" ) );

		m_circle_tolerance_slider = new QSlider( );
		m_circle_tolerance_slider->setRange( 0, 100 );
		m_circle_tolerance_slider->setValue( 20 );
		m_circle_tolerance_slider->setOrientation( Qt::Horizontal );
		connect( m_circle_tolerance_slider, SIGNAL( valueChanged( int ) ), this, SLOT( circle( int ) ) );
		
		m_circle_tolerance_lcd = new QLCDNumber( 3 );
		m_circle_tolerance_lcd->display( m_circle_tolerance_slider->value() ) ;

		m_circle_tolerance_label = new QLabel( tr( "Tolerance :" ) );

		
		m_line_layout = new QGridLayout();
		m_line_layout->addWidget( m_line_label, 0, 0 );
		m_line_layout->addWidget( m_line_slider, 0, 1 );
		m_line_layout->addWidget( m_line_lcd, 0, 2 );
		m_circle_layout = new QGridLayout();
		m_circle_layout->addWidget( m_circle_label, 0, 0 );
		m_circle_layout->addWidget( m_circle_slider, 0, 1 );
		m_circle_layout->addWidget( m_circle_lcd, 0, 2 );
		m_circle_layout->addWidget( m_circle_tolerance_label, 1, 0 );
		m_circle_layout->addWidget( m_circle_tolerance_slider, 1, 1 );
		m_circle_layout->addWidget( m_circle_tolerance_lcd, 1, 2 );
		
		m_line_widget = new QWidget();
		m_circle_widget = new QWidget();
		m_line_widget->setLayout( m_line_layout );
		m_circle_widget->setLayout( m_circle_layout );

		static_cast<QTabWidget*>(m_widget)->addTab( m_line_widget, tr("Lignes") );
		static_cast<QTabWidget*>(m_widget)->addTab( m_circle_widget, tr("Cercles") );

		connect( static_cast<QTabWidget*>(m_widget), SIGNAL( currentChanged( int ) ), this, SLOT( currentTabChanged( int ) ) );
	}
}
HoughWidget::~HoughWidget()
{
	if( m_hough ) delete m_hough;
}

void HoughWidget::init()
{
	if( m_hough ) currentTabChanged( 0 );
}

int HoughWidget::exec()
{
	if( !m_hough ) return QDialog::Rejected;
	else return QDialog::exec();
}

void HoughWidget::line( int )
{
	Image im;
	m_line_lcd->display( m_line_slider->value() ) ;
	start();
	m_hough->GetLines( &im, m_line_slider->value() );
	previewImage( im );
	stop();
}
void HoughWidget::circle( int )
{
	Image im;
	m_circle_lcd->display( m_circle_slider->value() ) ;
	m_circle_tolerance_lcd->display( m_circle_tolerance_slider->value() ) ;
	start();
	m_hough->GetCircles( &im, m_circle_slider->value(), m_circle_tolerance_slider->value() / 100.0f );
	previewImage( im );
	stop();
}

void HoughWidget::currentTabChanged( int index )
{
	switch( index )
	{
	case 0:
		line( m_line_slider->value() );
		break;
	case 1:
		circle( m_circle_slider->value() );
		break;
	}
}
