#include "histowidget.h"

HistoWidget::HistoWidget()
{
	m_marge = 5;
	setFixedSize( 256 + 2*m_marge, 256 + 2*m_marge );
	
	init();
	m_histo.setPixelNumber( 255 );
	m_histo.setMax( 1 );

	m_lastXChanged = 0;

}
HistoWidget::~HistoWidget()
{
	
}

void HistoWidget::activate()
{
	emit changed();
}

void HistoWidget::init()
{
	for( unsigned int i = 0 ; i < 256 ; ++i )
	{
		m_histo[ i ] = i;
	}
}

void HistoWidget::setParams( int luminosity, int contrast )
{
	int t[256];
	for( unsigned int i = 0 ; i < 128 ; ++i )
	{
		t[ i ] = 127 - (int)(tan( (89-contrast) * M_PI / 180.0 ) * (128-i)) - luminosity;
	}
	for( unsigned int i = 128 ; i < 256 ; ++i )
	{
		t[ i ] = 128 + (int)(tan( (89-contrast) * M_PI / 180.0 ) * (i-128)) - luminosity;
	}
	for( unsigned int i = 0 ; i < 255 ; ++i )
	{
		lineHisto( i, t[ i ], i+1, t[ i+1 ] );
	}
}

void HistoWidget::paintEvent( QPaintEvent* )
{
	QPainter p(this);
	p.fillRect( 0, 0, 256 + 2*m_marge, 256 + 2*m_marge, QBrush( Qt::black ) );
	p.fillRect( m_marge, m_marge, 256, 256, QBrush( Qt::white ) );
	
	p.setPen( Qt::black );
	for( unsigned int i = 0 ; i < 255 ; ++i )
	{
		p.drawLine( i + m_marge, 255-m_histo[ i ] + m_marge, i+1 + m_marge, 255-m_histo[ i+1 ] + m_marge );
	}
}

void HistoWidget::mouseMoveEvent( QMouseEvent* event )
{
	int x = event->x();
	int y = event->y();
	
	x -= m_marge;
	y -= m_marge;
	
	if( x > 255 ) x = 255;
	if( x < 0 ) x = 0;
	if( y > 255 ) y = 255;
	if( y < 0 ) y = 0;
	
	
	m_histo[ x ] = 255-y;
	
	lineHisto( m_lastXChanged, m_histo[ m_lastXChanged ], x, m_histo[ x ] );
	
	m_lastXChanged = x;
	
	update();
}
void HistoWidget::mousePressEvent( QMouseEvent* event )
{
	int x = event->x();
	int y = event->y();
	
	x -= m_marge;
	y -= m_marge;
	
	if( x > 255 ) x = 255;
	if( x < 0 ) x = 0;
	if( y > 255 ) y = 255;
	if( y < 0 ) y = 0;
	
	
	m_histo[ x ] = 255-y;
	
	m_lastXChanged = x;
	
	update();
}
void HistoWidget::mouseReleaseEvent( QMouseEvent* )
{
	for( unsigned int i = 0 ; i < 255 ; ++i )
	{
		if( m_histo[ i+1 ] < m_histo[ i ] ) m_histo[ i+1 ] = m_histo[ i ];
	}
	update();
	emit changed();
}

void HistoWidget::lineHisto( int x0, int y0, int x1, int y1 )
{
	bool steep = std::abs(y1 - y0) > std::abs(x1 - x0) ;
	if( steep )
	{
		std::swap( x0, y0 );
		std::swap( x1, y1 );
	}
	if( x0 > x1 )
	{
		std::swap( x0, x1 );
		std::swap( y0, y1 );
	}
	
	int deltax = x1 - x0 ;
	int deltay = abs(y1 - y0) ;
	int error = -(deltax + 1) / 2 ;
	int ystep ;
	int y = y0 ;
	ystep = ( y0 < y1 ? 1 : -1 );
	for( int x = x0 ; x <= x1 ; x++ )
	{
		if( steep )
		{
			if( y >= 0 && y < 256 )
				m_histo[y] = qBound( 0, x, 255 );
		}
		else
		{
			if( x >= 0 && x < 256 )
				m_histo[x] = qBound( 0, y, 255 );
		}
		error += deltay ;
		if( error >= 0 )
		{
			y += ystep ;
			error -= deltax ;
		}
	}	
}
