#include "specifwidget.h"

SpecifWidget::SpecifWidget( const Image& image )
	:	ToolWidget( new QWidget(), image )
{
	m_histo_widget = new HistoWidget();
	connect( m_histo_widget, SIGNAL( changed() ), this, SLOT( histoChanged() ) );

	m_local_CB = new QCheckBox( tr("Locale") );
	connect( m_local_CB, SIGNAL(stateChanged(int)), this, SLOT(stateLocalChanged(int)) );
	m_local_size_SB = new QSpinBox();
	m_local_size_SB->setRange( 1, qMin(image.GetHeight(), image.GetWidth())/2 - 1 );
	m_local_size_SB->setValue( m_local_size_SB->maximum() );
	m_local_size_SB->setEnabled( false );
	connect( m_local_size_SB, SIGNAL(valueChanged(int)), this, SLOT(valueLocalChanged(int)) );
	
	m_local_layout = new QHBoxLayout();
	m_local_layout->addWidget( m_local_CB );
	m_local_layout->addWidget( m_local_size_SB );
	
	
	m_lum_slider = new QSlider( );
	m_lum_slider->setRange( -255, 255 );
	m_lum_slider->setValue( 0 );
	m_lum_slider->setOrientation( Qt::Horizontal );
	connect( m_lum_slider, SIGNAL( valueChanged( int ) ), this, SLOT( luminosity( int ) ) );	
	m_contrast_slider = new QSlider( );
	m_contrast_slider->setRange( 0, 89 );
	m_contrast_slider->setValue( 45 );
	m_contrast_slider->setOrientation( Qt::Horizontal );
	connect( m_contrast_slider, SIGNAL( valueChanged( int ) ), this, SLOT( contrast( int ) ) );
	m_lum_lcd = new QLCDNumber( 3 );
	m_lum_lcd->display( m_lum_slider->value() ) ;
	m_contrast_lcd = new QLCDNumber( 3 );
	m_contrast_lcd->display( m_contrast_slider->value() ) ;
	m_lum_label = new QLabel( tr( "Luminosite :" ) );
	m_contrast_label = new QLabel( tr( "Contraste :" ) );

	m_sliders_layout = new QGridLayout();
	m_sliders_layout->addWidget( m_lum_label, 0, 0 );
	m_sliders_layout->addWidget( m_lum_slider, 1, 0 );
	m_sliders_layout->addWidget( m_lum_lcd, 1, 1 );
	m_sliders_layout->addWidget( m_contrast_label, 2, 0 );
	m_sliders_layout->addWidget( m_contrast_slider, 3, 0 );
	m_sliders_layout->addWidget( m_contrast_lcd, 3, 1 );
	
	m_right_layout = new QVBoxLayout();
	m_right_layout->addLayout( m_local_layout );
	m_right_layout->addStretch();
	m_right_layout->addLayout( m_sliders_layout );
	
	m_widget_layout = new QHBoxLayout();
	m_widget_layout->addWidget( m_histo_widget );
	m_widget_layout->addLayout( m_right_layout );
	
	m_widget->setLayout( m_widget_layout );
}
SpecifWidget::~SpecifWidget()
{
}

void SpecifWidget::init()
{
	m_histo_widget->activate();
}

void SpecifWidget::histoChanged()
{
	start();
	if( m_local_CB->checkState() == Qt::Checked )
	{
		previewImage( fast_specification_locale_MT( m_image, m_local_size_SB->value(), m_histo_widget->histo() ) );
	}
	else
	{
		Image im( m_image );
		specification( im, m_histo_widget->histo() );
		previewImage( im );
	}
	stop();
}
void SpecifWidget::stateLocalChanged( int state )
{
	m_local_size_SB->setEnabled( state == Qt::Checked );
	histoChanged();
}
void SpecifWidget::valueLocalChanged( int )
{
	histoChanged();
}

void SpecifWidget::luminosity( int value )
{
	m_lum_lcd->display( value ) ;
	m_histo_widget->setParams( value, m_contrast_slider->value() );
	histoChanged();
}
void SpecifWidget::contrast( int value )
{
	m_contrast_lcd->display( value ) ;
	m_histo_widget->setParams( m_lum_slider->value(), value );
	histoChanged();
}
