#include "previewwidget.h"

PreviewWidget::PreviewWidget( const Image& image )
{
	m_av_image.Copy( &image );
	m_ap_image.Copy( &image );
	
	m_av_label = new QLabel( tr("Avant") );
	m_ap_label = new QLabel( tr("Apres") );
	
	m_av_imageviewer = new ImageViewer( 256, 256 );
	m_av_imageviewer->setImage( m_av_image, 260, 260 );
	m_ap_imageviewer = new ImageViewer( 256, 256 );
	m_ap_imageviewer->setImage( m_ap_image, 260, 260 );
	
	connect( m_av_imageviewer, SIGNAL( scaleUpdated( double ) ), this, SLOT( updateApScale( double ) ) );
	connect( m_av_imageviewer, SIGNAL( positionUpdated( int, int ) ), this, SLOT( updateApPosition( int, int ) ) );
	connect( m_ap_imageviewer, SIGNAL( scaleUpdated( double ) ), this, SLOT( updateAvScale( double ) ) );
	connect( m_ap_imageviewer, SIGNAL( positionUpdated( int, int ) ), this, SLOT( updateAvPosition( int, int ) ) );
	

	m_layout = new QGridLayout();
	m_layout->addWidget( m_av_imageviewer, 0, 0 );
	m_layout->addWidget( m_av_label, 1, 0 );
	m_layout->addWidget( m_ap_imageviewer, 0, 1 );
	m_layout->addWidget( m_ap_label, 1, 1 );
	m_layout->setAlignment( m_av_label, Qt::AlignCenter );
	m_layout->setAlignment( m_ap_label, Qt::AlignCenter );
	//m_layout->setContentsMargins( 0, 0, 0, 0 );

	setLayout( m_layout );
	
	update();
}

PreviewWidget::~PreviewWidget()
{
}

void PreviewWidget::setImage( const Image& image )
{
	m_av_image.Copy( &image );
	m_av_imageviewer->setImage( m_av_image, 260, 260 );
	update();
}
const Image& PreviewWidget::image() const
{
	return m_av_image;
}
void PreviewWidget::setImagePreview( const Image& image )
{
	m_ap_image.Copy( &image );
	m_ap_imageviewer->setImage( m_ap_image, 260, 260 );
	update();
}
void PreviewWidget::updateImagePreview( const Image& image )
{
	m_ap_image.Copy( &image );
	update();
}
const Image& PreviewWidget::imagePreview() const
{
	return m_ap_image;
}

void PreviewWidget::update()
{
	Q_ASSERT( m_av_image.IsValid() && m_ap_image.IsValid() );
	//m_av_imageviewer->updateImage( m_av_image );
	m_ap_imageviewer->updateImage( m_ap_image );

}

void PreviewWidget::updateApScale( double scaleFactor )
{
	m_ap_imageviewer->updateScale( scaleFactor );
}
void PreviewWidget::updateApPosition( int x, int y )
{
	m_ap_imageviewer->updatePosition( x, y );
}
void PreviewWidget::updateAvScale( double scaleFactor )
{
	m_av_imageviewer->updateScale( scaleFactor );
}
void PreviewWidget::updateAvPosition( int x, int y )
{
	m_av_imageviewer->updatePosition( x, y );
}
