#include "toolwidget.h"

ToolWidget::ToolWidget( QWidget* widget, const Image& image, bool auto_apply )
{
	m_widget = widget;
	m_image.Copy( &image );
	
	m_preview_widget = new PreviewWidget( m_image );
	
	m_cb_auto_apply = new QCheckBox( tr("Appliquer automatiquement a l'image") );
	m_cb_auto_apply->setCheckState( auto_apply ? Qt::Checked : Qt::Unchecked );
	connect( m_cb_auto_apply, SIGNAL( stateChanged( int ) ), this, SLOT( applyAutoChanged( int ) ) );

	m_bt_apply = new QPushButton( tr("Appliquer") );
	connect( m_bt_apply, SIGNAL( clicked() ), this, SLOT( apply() ) );
	m_bt_cancel = new QPushButton( tr("Annuler") );
	connect( m_bt_cancel, SIGNAL( clicked() ), this, SLOT( cancel() ) );
	
	m_time_label = new QLabel();
	m_time_label->setFrameStyle( QFrame::StyledPanel );
	m_time_label->setFrameShadow( QFrame::Sunken );

	m_buttons_layout = new QHBoxLayout();
	m_buttons_layout->addWidget( m_time_label );
	m_buttons_layout->addWidget( m_bt_apply );
	m_buttons_layout->addWidget( m_bt_cancel );

	m_preview_layout = new QHBoxLayout();
	m_preview_layout->addWidget( m_preview_widget );


	m_preview_GB = new QGroupBox( tr( "Apercu" ) );
	m_preview_GB->setLayout( m_preview_layout );
	if( m_widget != NULL )
	{
		m_param_layout = new QHBoxLayout();
		m_param_layout->addWidget( m_widget );
		m_param_GB = new QGroupBox( tr( "Parametres" ) );
		m_param_GB->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Maximum );
		m_param_GB->setLayout( m_param_layout );
		m_param_GB->setContentsMargins( 0, m_param_GB->contentsRect().top(), 0, 0 );

	}
	m_preview_GB->setContentsMargins( 0, m_preview_GB->contentsRect().top(), 0, 0 );

	m_layout = new QVBoxLayout();
	m_layout->addWidget( m_preview_GB );
	if( m_widget != NULL ) m_layout->addWidget( m_param_GB );
	m_layout->addWidget( m_cb_auto_apply );
	m_layout->addLayout( m_buttons_layout );
	
	setLayout( m_layout );

	setWindowFlags( Qt::WindowMinMaxButtonsHint );

	connect( this, SIGNAL( finished(int) ), this, SLOT( finish(int) ) );
}
ToolWidget::~ToolWidget()
{
}

void ToolWidget::init()
{

}

void ToolWidget::apply()
{
	done( 0 );
}
void ToolWidget::cancel()
{
	done( 1 );
}
void ToolWidget::finish( int result )
{
	if( result == 0 )
	{
		emit applyImage( m_preview_widget->imagePreview() );
	}
	else
	{
		emit updateImage( m_preview_widget->image() );
	}
}

void ToolWidget::start()
{
	qApp->setOverrideCursor( Qt::WaitCursor );
	m_time_label->setText( tr( "Veuillez patienter..." ) );
	repaint();
	m_time.start();
}
void ToolWidget::stop()
{
	m_time_label->setText( tr( "Temps : %1 s" ).arg( m_time.elapsed() / 1000.0 ) );
	qApp->restoreOverrideCursor();
}

void ToolWidget::previewImage( const Image& image )
{
	m_preview_widget->updateImagePreview( image );
	if( m_cb_auto_apply->checkState() == Qt::Checked )
	{
		emit updateImage( image );
	}
}

void ToolWidget::applyAutoChanged( int state )
{
	if( state == Qt::Checked )
	{
		emit updateImage( m_preview_widget->imagePreview() );
	}
	else
	{
		emit updateImage( m_preview_widget->image() );
	}
}

void ToolWidget::closeEvent( QCloseEvent * )
{
	done( 1 );
}