#include "imagelabel.h"

ImageLabel::ImageLabel()
{
	resize( 100, 20 ); //Pour le texte "Pas d'image"
	m_scaleFactor = 1.0;
}
ImageLabel::~ImageLabel()
{
}

void ImageLabel::retranslate()
{
	repaint();
}

void ImageLabel::setImage( const Image& _image, double scaleFactor )
{
	m_image.Copy( &_image );
	m_scaleFactor = scaleFactor;
	
	update();
}

void ImageLabel::updateImage( const Image& _image )
{
	m_image.Copy( &_image );
	update();
}
void ImageLabel::updateScale( double scaleFactor )
{
	m_scaleFactor = scaleFactor;
	resize( m_scaleFactor * m_pixmap.size() );
}

void ImageLabel::update()
{
	if( m_image.IsValid() )
	{
		QImage img( m_image.GetWidth(), m_image.GetHeight(), QImage::Format_RGB32 );
		if( m_image.GetChanNumber() == 3 )
		{
			for( unsigned int i = 0 ; i < m_image.GetHeight() ; ++i )
			{
				for( unsigned int j = 0 ; j < m_image.GetWidth() ; ++j )
				{
					unsigned char* c = m_image.GetPixel( j, i );
					img.setPixel( j, i, qRgb( c[0], c[1], c[2] ) );
				}
			}
		}
		else
		{
			for( unsigned int i = 0 ; i < m_image.GetHeight() ; ++i )
			{
				for( unsigned int j = 0 ; j < m_image.GetWidth() ; ++j )
				{
					unsigned char* c = m_image.GetPixel( j, i );
					img.setPixel( j, i, qRgb( c[0], c[0], c[0] ) );
				}
			}
		}
		m_pixmap = QPixmap::fromImage( img );
		resize( m_scaleFactor * m_pixmap.size() );
		repaint();
	}	
}

void ImageLabel::paintEvent( QPaintEvent * )
{
	int xoffset = 0, yoffset = 0;
	if( width() > m_pixmap.width()*m_scaleFactor )
		xoffset = (int) (width()-m_pixmap.width()*m_scaleFactor)/2;
	if( height() > m_pixmap.height()*m_scaleFactor )
		yoffset = (int) (height()-m_pixmap.height()*m_scaleFactor)/2;

	if( m_image.IsValid() )
	{
		QPainter p( this );
		if( m_scaleFactor < 1.0 )
			p.setRenderHint( QPainter::SmoothPixmapTransform );
		p.save();
		p.translate( xoffset, yoffset );
		p.scale( m_scaleFactor, m_scaleFactor );
		p.drawPixmap( 0, 0, m_pixmap );
		p.restore();
		
	}
	else
	{
		QPainter p( this );
		p.drawText( 0 , 0, width(), height(), Qt::AlignCenter, tr( "Pas d'image." ) );
	}
}

void ImageLabel::wheelEvent( QWheelEvent *event )
{
	if( m_image.IsValid() )
	{
		int numDegrees = event->delta() / 8;
		int numSteps = numDegrees / 15;

		if (event->orientation() == Qt::Vertical)
		{
			if( numSteps < 0 )
			{
				if( m_scaleFactor > 1.0/16.0 )
				{
					m_scaleFactor *= 0.5;
					emit updated( m_scaleFactor, 0.5, event->x(), event->y() );
					//resize( m_scaleFactor * m_pixmap.size() );
				}
			}
			else
			{
				if( m_scaleFactor < 1.0*32.0 )
				{
					m_scaleFactor *= 2.0;
					//resize( m_scaleFactor * m_pixmap.size() );
					emit updated( m_scaleFactor, 2.0, event->x(), event->y() );
				}
			}
			repaint();
			event->accept();
		}
	}
}
