#include "imageviewer.h"

ImageViewer::ImageViewer( unsigned int width, unsigned int height )
{
	m_imageLabel = new ImageLabel();
	connect( m_imageLabel, SIGNAL( updated( double, double, int, int ) ), this, SLOT( update( double, double, int, int ) ) );
	
	m_scrollArea = new QScrollArea();
	m_scrollArea->setBackgroundRole(QPalette::Dark);
	m_scrollArea->setAlignment( Qt::AlignCenter );
	m_scrollArea->setMinimumSize( width+4, height+4 );
	m_scrollArea->setWidget( m_imageLabel );

	connect( m_scrollArea->horizontalScrollBar(), SIGNAL( valueChanged( int ) ), this, SLOT( scrollBarValueChange( int ) ) );
	connect( m_scrollArea->verticalScrollBar(), SIGNAL( valueChanged( int ) ), this, SLOT( scrollBarValueChange( int ) ) );

	m_layout = new QVBoxLayout();
	m_layout->addWidget( m_scrollArea );
	//m_layout->setContentsMargins( 0, 0, 0, 0 );
	setLayout( m_layout );
}
ImageViewer::~ImageViewer()
{
}

void ImageViewer::retranslate()
{
	m_imageLabel->retranslate();
}

void ImageViewer::setImage( const Image& _image, unsigned int preferred_width, unsigned int preferred_height )
{
	double scaleFactor = 1.0;
	if( preferred_width == 0 || preferred_height == 0 )
	{
		preferred_width = width();
		preferred_height = height();
	}
	while( _image.GetHeight() * scaleFactor > (preferred_height-4) || _image.GetWidth() * scaleFactor > (preferred_width-4) ) scaleFactor *= 0.5;
	while( _image.GetHeight() * scaleFactor * 2.0 <= (preferred_height-4) && _image.GetWidth() * scaleFactor * 2.0 <= (preferred_width-4) ) scaleFactor *= 2.0;

	m_imageLabel->setImage( _image, scaleFactor );
}
void ImageViewer::updateImage( const Image& _image )
{
	m_imageLabel->updateImage( _image );
}

void ImageViewer::updateScale( double scaleFactor )
{
	disconnect( m_scrollArea->horizontalScrollBar(), SIGNAL( valueChanged( int ) ), this, SLOT( scrollBarValueChange( int ) ) );
	disconnect( m_scrollArea->verticalScrollBar(), SIGNAL( valueChanged( int ) ), this, SLOT( scrollBarValueChange( int ) ) );
	m_imageLabel->updateScale( scaleFactor );
	connect( m_scrollArea->horizontalScrollBar(), SIGNAL( valueChanged( int ) ), this, SLOT( scrollBarValueChange( int ) ) );
	connect( m_scrollArea->verticalScrollBar(), SIGNAL( valueChanged( int ) ), this, SLOT( scrollBarValueChange( int ) ) );	
}
void ImageViewer::updatePosition( int scroll_x, int scroll_y )
{
	disconnect( m_scrollArea->horizontalScrollBar(), SIGNAL( valueChanged( int ) ), this, SLOT( scrollBarValueChange( int ) ) );
	disconnect( m_scrollArea->verticalScrollBar(), SIGNAL( valueChanged( int ) ), this, SLOT( scrollBarValueChange( int ) ) );
	QScrollBar* scrollBar;
	scrollBar = m_scrollArea->horizontalScrollBar();
	scrollBar->setValue( scroll_x );
	scrollBar = m_scrollArea->verticalScrollBar();
	scrollBar->setValue( scroll_y );
	connect( m_scrollArea->horizontalScrollBar(), SIGNAL( valueChanged( int ) ), this, SLOT( scrollBarValueChange( int ) ) );
	connect( m_scrollArea->verticalScrollBar(), SIGNAL( valueChanged( int ) ), this, SLOT( scrollBarValueChange( int ) ) );
}

void ImageViewer::update( double scaleFactor, double scale, int x, int y )
{
	int scroll_x, scroll_y;
	QScrollBar *x_scrollBar, *y_scrollBar;
	x_scrollBar = m_scrollArea->horizontalScrollBar();
	y_scrollBar = m_scrollArea->verticalScrollBar();
	
	scroll_x = (int) (x * scale - (x-x_scrollBar->value()) );
	scroll_y = (int) (y * scale - (y-y_scrollBar->value()) );
	
	//Version centre
	//scroll_x =(int)( (x*scale) ) - width()/2;
	//scroll_y =(int)( (y*scale) ) - height()/2;
	
	updateScale( scaleFactor );
	updatePosition( scroll_x, scroll_y );

	emit scaleUpdated( scaleFactor );
	emit positionUpdated( m_scrollArea->horizontalScrollBar()->value(), m_scrollArea->verticalScrollBar()->value() );
}

void ImageViewer::scrollBarValueChange( int )
{
	emit positionUpdated( m_scrollArea->horizontalScrollBar()->value(), m_scrollArea->verticalScrollBar()->value() );
}

void ImageViewer::mouseMoveEvent( QMouseEvent * event )
{
	//if( event->buttons() != Qt::NoButton )
	{
		int x_delta = event->x() - m_x_save;
		int y_delta = event->y() - m_y_save;
		m_x_save = event->x();
		m_y_save = event->y();

		updatePosition( m_scrollArea->horizontalScrollBar()->value() - x_delta, m_scrollArea->verticalScrollBar()->value() - y_delta );

		emit positionUpdated( m_scrollArea->horizontalScrollBar()->value(), m_scrollArea->verticalScrollBar()->value() );
		
		event->accept();
	}
}
void ImageViewer::mousePressEvent( QMouseEvent * event )
{
	m_x_save = event->x();
	m_y_save = event->y();
}
