#ifndef _IMAGEVIEWER_H_
#define _IMAGEVIEWER_H_

#include <QtGui>
#include <QtCore>

#include "imagelabel.h"

class ImageViewer : public QWidget
{
	Q_OBJECT
public:
	ImageViewer( unsigned int width = 512, unsigned int height = 512 );
	~ImageViewer();

	const double scaleFactor() const { return m_imageLabel->scaleFactor(); }
	const Image& image() const { return m_imageLabel->image(); }
	void position( int* x, int* y ) const { *x = m_scrollArea->horizontalScrollBar()->value(); *y = m_scrollArea->verticalScrollBar()->value(); }

	void setImage( const Image& _image, unsigned int preferred_width = 0, unsigned int preferred_height = 0 );
	
	void retranslate();

signals:
	void scaleUpdated( double scaleFactor );
	void positionUpdated( int x, int y );

public slots:
	void updateImage( const Image& _image );
	void updateScale( double scaleFactor ); 
	void updatePosition( int scroll_x, int scroll_y );

private slots:
	void update( double scaleFactor, double scale, int x, int y );
	void scrollBarValueChange( int );

private:
	ImageLabel *m_imageLabel;
	QScrollArea *m_scrollArea;

	int m_x_save;
	int m_y_save;

	QVBoxLayout *m_layout;

protected:
	virtual void mouseMoveEvent( QMouseEvent * event );
	virtual void mousePressEvent( QMouseEvent * event );

};

#endif
