#include "mainwindow.h"

MainWindow::MainWindow()
{
	m_mainWidget = new QWidget();
	setCentralWidget( m_mainWidget );
	
	m_imageViewer = new ImageViewer();
	connect( m_imageViewer, SIGNAL( scaleUpdated( double ) ), this, SLOT( updateScaleFactor( double ) ) );
	
	m_nameLabel = new QLabel("");
	m_nameLabel->setFrameStyle( QFrame::StyledPanel );
	m_nameLabel->setFrameShadow( QFrame::Sunken );
	m_sizeLabel = new QLabel("");
	m_sizeLabel->setFrameStyle( QFrame::StyledPanel );
	m_sizeLabel->setFrameShadow( QFrame::Sunken );
	m_zoomLabel = new QLabel("");
	m_zoomLabel->setFrameStyle( QFrame::StyledPanel );
	m_zoomLabel->setFrameShadow( QFrame::Sunken );
	
	m_labelsLayout = new QHBoxLayout();
	m_labelsLayout->addWidget( m_nameLabel );
	m_labelsLayout->addWidget( m_sizeLabel );
	m_labelsLayout->addWidget( m_zoomLabel );
	m_labelsLayout->addStretch(1);

	m_layout = new QVBoxLayout();
	m_layout->addWidget( m_imageViewer, Qt::AlignCenter );
	m_layout->addLayout( m_labelsLayout );
	m_mainWidget->setLayout( m_layout );

	m_dir = ".";
	m_history_index = -1;

	createActions();
	createMenus();

	if( qApp->arguments().size() > 1 )
	{
		openFile( qApp->arguments()[1] );
	}
	
	m_translator = new QTranslator();
	m_translator->load("translations/gui_fr.qm");
	qApp->installTranslator( m_translator );
	m_global_translator = new QTranslator();
    m_global_translator->load("qt_fr", QLibraryInfo::location(QLibraryInfo::TranslationsPath));
    qApp->installTranslator(m_global_translator);
	
	retranslate();
	
	
	update();
	
	setAcceptDrops(true);
}
MainWindow::~MainWindow()
{
}

void MainWindow::createMenus()
{
	m_fileMenu = menuBar()->addMenu("Fichier");
	m_fileMenu->addAction(m_openAct);
	m_fileMenu->addAction(m_saveAct);
	m_fileMenu->addSeparator();
	m_fileMenu->addAction(m_exitAct);

	m_editMenu =  menuBar()->addMenu("Edition");
	m_editMenu->addAction( m_undoAct );
	m_editMenu->addAction( m_redoAct );
	
	m_toolsMenu =  menuBar()->addMenu("Outils");
	m_toolsMenu->addAction( m_filterAct );
	m_toolsMenu->addAction( m_specifAct );
	m_toolsMenu->addAction( m_kmeansAct );
	m_toolsMenu->addAction( m_contourAct );
	m_toolsMenu->addAction( m_tramageAct );
	m_toolsMenu->addAction( m_fourierAct );
	m_toolsMenu->addAction( m_houghAct );
	
	m_languageMenu =  new QMenu("Langue");
	m_languageMenu->addAction( m_frAct );
	m_languageMenu->addAction( m_enAct );
	
	m_helpMenu =  menuBar()->addMenu( "?" );
	m_helpMenu->addMenu( m_languageMenu );
	
}
void MainWindow::createActions()
{
	m_openAct = new QAction("", this);
	connect(m_openAct, SIGNAL(triggered()), this, SLOT(open()));
	
	m_saveAct = new QAction("", this);
	m_saveAct->setEnabled( false );
	connect(m_saveAct, SIGNAL(triggered()), this, SLOT(save()));
	
	m_exitAct = new QAction("", this);
	connect(m_exitAct, SIGNAL(triggered()), qApp, SLOT(quit()));

	m_undoAct = new QAction("", this);
	m_undoAct->setEnabled( false );
	connect(m_undoAct, SIGNAL(triggered()), this, SLOT(undo()));
	m_redoAct = new QAction("", this);
	m_redoAct->setEnabled( false );
	connect(m_redoAct, SIGNAL(triggered()), this, SLOT(redo()));

	m_frAct = new QAction("", this);
	m_frAct->setCheckable( true );
	m_frAct->setChecked( true );
	connect(m_frAct, SIGNAL(triggered()), this, SLOT(setLanguageFr()));
	m_enAct = new QAction("", this);
	m_enAct->setCheckable( true );
	m_enAct->setChecked( false );
	connect(m_enAct, SIGNAL(triggered()), this, SLOT(setLanguageEn()));

	m_filterAct = new QAction("", this );
	connect(m_filterAct, SIGNAL(triggered()), this, SLOT(filter()));
	
	m_specifAct = new QAction("", this );
	connect(m_specifAct, SIGNAL(triggered()), this, SLOT(specification()));
	
	m_kmeansAct = new QAction("", this );
	connect(m_kmeansAct, SIGNAL(triggered()), this, SLOT(kmeans()));
	
	m_contourAct = new QAction("", this );
	connect(m_contourAct, SIGNAL(triggered()), this, SLOT(contour()));

	m_tramageAct = new QAction("", this );
	connect(m_tramageAct, SIGNAL(triggered()), this, SLOT(tramage()));

	m_fourierAct = new QAction("", this );
	connect(m_fourierAct, SIGNAL(triggered()), this, SLOT(fourier()));

	m_houghAct = new QAction("", this );
	connect(m_houghAct, SIGNAL(triggered()), this, SLOT(hough()));

}

void MainWindow::retranslate()
{
	m_fileMenu->setTitle( tr("&Fichier") );
	m_editMenu->setTitle( tr("&Edition") );
	m_toolsMenu->setTitle( tr("&Outils") );
	m_languageMenu->setTitle( tr("&Langue") );
	
	m_openAct->setText( tr("&Ouvrir...") );
	m_openAct->setShortcut(tr("Ctrl+O"));
	m_saveAct->setText(tr("&Sauvegarder..."));
	m_saveAct->setShortcut(tr("Ctrl+S"));
	m_exitAct->setText(tr("&Quitter"));
	m_exitAct->setShortcut(tr("Ctrl+Q"));
	m_undoAct->setText(tr("&Annuler"));
	m_undoAct->setShortcut(tr("Ctrl+Z"));
	m_redoAct->setText(tr("&Refaire"));
	m_redoAct->setShortcut(tr("Ctrl+Shift+Z"));
	
	m_frAct->setText(tr("&Francais"));
	m_enAct->setText(tr("&English"));
	m_filterAct->setText( tr("&Filtres...") );
	m_specifAct->setText( tr("&Specification...") );
	m_kmeansAct->setText( tr("&KMoyennes...") );
	m_contourAct->setText( tr("&Contours...") );
	m_tramageAct->setText( tr("&Tramage...") );
	m_fourierAct->setText( tr("&Fourier...") );
	m_houghAct->setText( tr("&Hough...") );
	
	m_imageViewer->retranslate();
}

void MainWindow::open()
{
	QString str = QFileDialog::getOpenFileName(this,tr("Ouvrir une image"), m_dir,tr("Tous fichiers (*.*);;JPEG (*.jpg *.jpeg);;Portable Network Graphics (*.png);;Portable Image (*.pnm *.ppm *.pgm *.pbm)"));
	if ( str != QString::null )
	{
		m_dir = str;
		openFile( str );
	}
}
void MainWindow::save()
{
	if( !m_imageViewer->image().IsValid() ) return;

	QString str = QFileDialog::getSaveFileName(this,tr("Enregistrer l'image"), m_dir,tr("JPEG (*.jpg *.jpeg);;Portable Network Graphics (*.png);;Portable Image (*.pnm *.ppm *.pgm *.pbm)"));
	if ( str != QString::null )
	{
		m_dir = str;
		
		QFileInfo fileInfo( str );
		QString ext = fileInfo.suffix().toLower();
		bool result = false;

		if( ext == "jpg" || ext == "jpeg" )
		{
			result = m_imageViewer->image().SaveJPG( str.toAscii().data() );
		}
		else if ( ext == "png" )
		{
			result = m_imageViewer->image().SavePNG( str.toAscii().data() );
		}
		else if ( ext == "pnm" || ext == "ppm" )
		{
			result = m_imageViewer->image().SavePNM( str.toAscii().data() );
		}
		else if ( ext == "pgm" )
		{
			result = m_imageViewer->image().SavePNM( str.toAscii().data(), Image::PGM_BINARY );
		}
		else if ( ext == "pbm" )
		{
			result = m_imageViewer->image().SavePNM( str.toAscii().data(), Image::PBM_BINARY );
		}

		if( !result )
			QMessageBox::warning( this, tr("Erreur"), tr("Erreur lors de la sauvegarde.") );
	}
}
void MainWindow::openFile( const QString filename )
{
	Image im;

	QFileInfo fileInfo( filename );
	QString ext = fileInfo.suffix().toLower();

	if( ext == "jpg" || ext == "jpeg" )
	{
		im.Load( filename.toAscii().data(), Image::JPG );
	}
	else if ( ext == "png" )
	{
		im.Load( filename.toAscii().data(), Image::PNG );
	}
	else if ( ext == "pnm" || ext == "ppm" || ext == "pgm" || ext == "pbm" )
	{
		im.Load( filename.toAscii().data(), Image::PNM );
	}
	else
	{
		QMessageBox::warning( this, tr("Erreur"), tr("Format d'image non reconnu.") );
		return;
	}

	if( im.IsValid() )
	{
		m_saveAct->setEnabled( true );
		m_nameLabel->setText( fileInfo.fileName() );
		m_imageViewer->setImage( im );
		m_images_history.clear();
		m_images_history.push_back( im );
		m_history_index = 0;
		m_undoAct->setEnabled( false );
		m_redoAct->setEnabled( false );
		
		update();
	}
	else
	{
		QMessageBox::warning( this, tr("Erreur"), tr("Impossible d'ouvrir ce fichier.") );
		return;
	}
}

void MainWindow::undo()
{
	m_history_index--;
	Q_ASSERT( m_history_index > 0 );
	m_imageViewer->updateImage( m_images_history[m_history_index] );
	m_redoAct->setEnabled( true );
	if( m_history_index == 0 )
		m_undoAct->setEnabled( false );
}
void MainWindow::redo()
{
	m_history_index++;
	Q_ASSERT( m_history_index < m_images_history.size() );
	m_imageViewer->updateImage( m_images_history[m_history_index] );
	m_undoAct->setEnabled( true );
	if( m_history_index == m_images_history.size()-1 )
		m_redoAct->setEnabled( false );
}


void MainWindow::filter()
{
	FilterWidget *w = new FilterWidget( m_imageViewer->image() );
	w->setWindowTitle( tr( "Application de filtres" ) );
	connect( w, SIGNAL( updateImage( const Image& ) ), this, SLOT( updateImage( const Image& ) ) );
	connect( w, SIGNAL( applyImage( const Image& ) ), this, SLOT( applyImage( const Image& ) ) );
	w->init();
	w->exec();
	disconnect( w, SIGNAL( updateImage( const Image& ) ), this, SLOT( updateImage( const Image& ) ) );
	disconnect( w, SIGNAL( applyImage( const Image& ) ), this, SLOT( applyImage( const Image& ) ) );
	delete w;
}
void MainWindow::specification()
{
	SpecifWidget *w = new SpecifWidget( m_imageViewer->image() );
	w->setWindowTitle( tr( "Specification d'histogramme" ) );
	connect( w, SIGNAL( updateImage( const Image& ) ), this, SLOT( updateImage( const Image& ) ) );
	connect( w, SIGNAL( applyImage( const Image& ) ), this, SLOT( applyImage( const Image& ) ) );
	w->init();
	w->exec();
	disconnect( w, SIGNAL( updateImage( const Image& ) ), this, SLOT( updateImage( const Image& ) ) );
	disconnect( w, SIGNAL( applyImage( const Image& ) ), this, SLOT( applyImage( const Image& ) ) );
	delete w;
}

void MainWindow::contour()
{
	ContourWidget *w = new ContourWidget( m_imageViewer->image() );
	w->setWindowTitle( tr( "Detection de contours" ) );
	connect( w, SIGNAL( updateImage( const Image& ) ), this, SLOT( updateImage( const Image& ) ) );
	connect( w, SIGNAL( applyImage( const Image& ) ), this, SLOT( applyImage( const Image& ) ) );
	w->init();
	w->exec();
	disconnect( w, SIGNAL( updateImage( const Image& ) ), this, SLOT( updateImage( const Image& ) ) );
	disconnect( w, SIGNAL( applyImage( const Image& ) ), this, SLOT( applyImage( const Image& ) ) );
	delete w;
}
void MainWindow::tramage()
{
	TramageWidget *w = new TramageWidget( m_imageViewer->image() );
	w->setWindowTitle( tr( "Tramage" ) );
	connect( w, SIGNAL( updateImage( const Image& ) ), this, SLOT( updateImage( const Image& ) ) );
	connect( w, SIGNAL( applyImage( const Image& ) ), this, SLOT( applyImage( const Image& ) ) );
	w->init();
	w->exec();
	disconnect( w, SIGNAL( updateImage( const Image& ) ), this, SLOT( updateImage( const Image& ) ) );
	disconnect( w, SIGNAL( applyImage( const Image& ) ), this, SLOT( applyImage( const Image& ) ) );
	delete w;
}

void MainWindow::fourier()
{
	FourierWidget *w = new FourierWidget( m_imageViewer->image() );
	w->setWindowTitle( tr( "Fourier" ) );
	connect( w, SIGNAL( updateImage( const Image& ) ), this, SLOT( updateImage( const Image& ) ) );
	connect( w, SIGNAL( applyImage( const Image& ) ), this, SLOT( applyImage( const Image& ) ) );
	w->init();
	w->exec();
	disconnect( w, SIGNAL( updateImage( const Image& ) ), this, SLOT( updateImage( const Image& ) ) );
	disconnect( w, SIGNAL( applyImage( const Image& ) ), this, SLOT( applyImage( const Image& ) ) );
	delete w;
}

void MainWindow::hough()
{
	HoughWidget *w = new HoughWidget( m_imageViewer->image() );
	w->setWindowTitle( tr( "Hough" ) );
	connect( w, SIGNAL( updateImage( const Image& ) ), this, SLOT( updateImage( const Image& ) ) );
	connect( w, SIGNAL( applyImage( const Image& ) ), this, SLOT( applyImage( const Image& ) ) );
	w->init();
	w->exec();
	disconnect( w, SIGNAL( updateImage( const Image& ) ), this, SLOT( updateImage( const Image& ) ) );
	disconnect( w, SIGNAL( applyImage( const Image& ) ), this, SLOT( applyImage( const Image& ) ) );
	delete w;
}

void MainWindow::kmeans()
{
	KMeansWidget *w = new KMeansWidget( m_imageViewer->image() );
	w->setWindowTitle( tr( "KMoyennes" ) );
	connect( w, SIGNAL( updateImage( const Image& ) ), this, SLOT( updateImage( const Image& ) ) );
	connect( w, SIGNAL( applyImage( const Image& ) ), this, SLOT( applyImage( const Image& ) ) );
	w->init();
	w->exec();
	disconnect( w, SIGNAL( updateImage( const Image& ) ), this, SLOT( updateImage( const Image& ) ) );
	disconnect( w, SIGNAL( applyImage( const Image& ) ), this, SLOT( applyImage( const Image& ) ) );
	delete w;
}

void MainWindow::applyImage( const Image& image )
{
	m_imageViewer->updateImage( image );
	while( m_history_index < m_images_history.size()-1 ) m_images_history.removeLast();
	m_images_history.push_back( image );
	m_history_index++;
	m_undoAct->setEnabled( true );
	m_redoAct->setEnabled( false );
	update();
}
void MainWindow::updateImage( const Image& image )
{
	m_imageViewer->updateImage( image );
	update();
}

void MainWindow::updateScaleFactor( double scaleFactor )
{
	m_zoomLabel->setText( tr("%1%").arg( int( scaleFactor * 100.0 ) ) );
}

void MainWindow::setLanguageFr()
{
	if( QFile::exists( "translations/gui_fr.qm") )
	{
		qApp->removeTranslator( m_translator );
		m_translator->load("translations/gui_fr.qm");
		qApp->installTranslator( m_translator );
		qApp->removeTranslator( m_global_translator );
		m_global_translator->load("qt_fr", QLibraryInfo::location(QLibraryInfo::TranslationsPath));
		qApp->installTranslator(m_global_translator);
		retranslate();
		m_frAct->setChecked( true );
		m_enAct->setChecked( false );
	}
}
void MainWindow::setLanguageEn()
{
	if( QFile::exists( "translations/gui_en.qm") )
	{
		qApp->removeTranslator( m_translator );
		m_translator->load("translations/gui_en.qm");
		qApp->installTranslator( m_translator );
		qApp->removeTranslator( m_global_translator );
		m_global_translator->load("qt_en", QLibraryInfo::location(QLibraryInfo::TranslationsPath));
		qApp->installTranslator(m_global_translator);
		retranslate();
		m_frAct->setChecked( false );
		m_enAct->setChecked( true );
	}
}

void MainWindow::update()
{
	if( m_imageViewer->image().IsValid() )
	{
		m_sizeLabel->setText( tr("%1x%2x%3").arg(m_imageViewer->image().GetHeight()).arg(m_imageViewer->image().GetWidth()).arg(m_imageViewer->image().GetChanNumber()*8) );
		m_zoomLabel->setText( tr("%1%").arg( int(m_imageViewer->scaleFactor()*100.0) ) );

		m_specifAct->setEnabled( true );
		m_filterAct->setEnabled( true );
		m_contourAct->setEnabled( true );
		m_tramageAct->setEnabled( true );
		m_fourierAct->setEnabled( true );
		m_houghAct->setEnabled( true );
		m_kmeansAct->setEnabled( true );
	}
	else
	{
		m_specifAct->setEnabled( false );
		m_filterAct->setEnabled( false );
		m_contourAct->setEnabled( false );
		m_tramageAct->setEnabled( false );
		m_fourierAct->setEnabled( false );
		m_houghAct->setEnabled( false );
		m_kmeansAct->setEnabled( false );
	}
}

void MainWindow::dragEnterEvent( QDragEnterEvent *event )
{
	if( event->mimeData()->hasFormat("text/uri-list") || event->mimeData()->hasImage() ) 
	{     
		event->acceptProposedAction();
	}
}
void MainWindow::dropEvent( QDropEvent * event )
{
	QList<QUrl> urls;
    QString file_name;
    QFileInfo info;
 
    if( event->mimeData()->hasUrls() )
    {
        urls = event->mimeData()->urls();
		
		if( urls.size() > 0 )
        {
            file_name = urls[0].toLocalFile();
            info.setFile( file_name );
            if( info.isFile() )
			{
				openFile( file_name );
				event->acceptProposedAction();
			}
        }
    }
}
