#ifndef _MAINWINDOW_H_
#define _MAINWINDOW_H_

#include <QtGui>
#include <QtCore>

#include <ImageUtils.h>

#include "imageviewer.h"
#include "Outils/Specification/specifwidget.h"
#include "Outils/Filtres/filterwidget.h"
#include "Outils/Contours/contourwidget.h"
#include "Outils/Tramage/tramagewidget.h"
#include "Outils/Fourier/fourierwidget.h"
#include "Outils/Hough/houghwidget.h"
#include "Outils/KMoyennes/kmeanswidget.h"

class MainWindow : public QMainWindow
{
	Q_OBJECT
public:
	MainWindow();
	~MainWindow();

	void retranslate();
	
public slots:
	void open();
	void save();
	void undo();
	void redo();

	void filter();
	void specification();
	void contour();
	void tramage();
	void fourier();
	void hough();
	void kmeans();
	
	void applyImage( const Image& image );
	void updateImage( const Image& image );

private slots:
	void updateScaleFactor( double scaleFactor );
	
	void setLanguageFr();
	void setLanguageEn();

private:
	QString m_dir;
	QTranslator *m_translator, *m_global_translator;
	QList<Image> m_images_history;
	int m_history_index;

	/*** Interface ***/
	QWidget *m_mainWidget;
	ImageViewer* m_imageViewer;
	QLabel *m_nameLabel, *m_sizeLabel, *m_zoomLabel;
	
     
	QMenu *m_fileMenu, *m_editMenu, *m_toolsMenu, *m_helpMenu, *m_languageMenu;
	QAction *m_openAct, *m_saveAct, *m_exitAct;
	QAction *m_undoAct, *m_redoAct;
	QAction *m_filterAct, *m_specifAct, *m_contourAct, *m_tramageAct, *m_fourierAct, *m_houghAct, *m_kmeansAct;
	QAction *m_frAct, *m_enAct;

	QHBoxLayout *m_labelsLayout;
	QVBoxLayout *m_layout;
	
	void createMenus();
	void createActions();

	void openFile( const QString filename );

	void update();
	
protected:
	virtual void dragEnterEvent ( QDragEnterEvent * event );
	virtual void dropEvent ( QDropEvent * event );
	
};

#endif //_MAINWINDOW_H_
