#pragma once


class Image
{
public:

	enum Format
	{
		UNKNOWN = -1,
		PBM     = 0,
		PGM,
		PPM,
		PNM,
		BMP,
		TGA,
		JPG,
		PNG,

		FormatNumber
	};

	enum FormatOption
	{
		NONE = Image::FormatNumber + 1,
		PBM_ASCII,
		PBM_BINARY,
		PGM_ASCII,
		PGM_BINARY,
		PPM_ASCII,
		PPM_BINARY,
		PNG_INTERLACED,

		FormatOptionNumber
	};


	Image( void );
	Image( const Image & );
	Image( const char *filename, Image::Format );
	Image( unsigned int width, unsigned int height, unsigned char chan_number );
	~Image( void );

	bool Create( unsigned int width, unsigned int height, unsigned char chan_number );
	bool Copy ( const Image * );
	bool Load ( const char *filename, Image::Format );
	bool Save ( const char *filename, Image::Format ) const;
	void Clear( void );

	bool IsValid( void ) const;

	unsigned char * GetPixel( unsigned int i )                 const;
	unsigned char * GetPixel( unsigned int x, unsigned int y ) const;

	unsigned int  GetWidth(      void ) const;
	unsigned int  GetHeight(     void ) const;
	unsigned char GetChanNumber( void ) const;

	bool LoadPNM( const char *filename );
	bool LoadBMP( const char *filename );
	bool LoadJPG( const char *filename );
	bool LoadPNG( const char *filename );

	bool SavePNM( const char *filename, FormatOption option = PPM_BINARY ) const;
	bool SaveBMP( const char *filename                                   ) const;
	bool SaveJPG( const char *filename, int         quality = 80         ) const;
	bool SavePNG( const char *filename, FormatOption option = NONE       ) const;


protected:


	unsigned int   m_width;       // Largeur en pixels
	unsigned int   m_height;      // Hauteur en pixels

	unsigned char *m_pixels;      // Contenu des donnes

	unsigned char  m_chan_number; // Nombre de canaux
	unsigned char  m_depth;       // Nombre d'octets par canaux

	// Donnes utiles
	unsigned int   m_pixel_number;
	unsigned char  m_chan_mult_depth;

	// Met  jour des donnes utiles
	void Update( void );
};
