#include "image_util.h"

#include <stdio.h>
#include <string.h>




// Gestion des erreurs
void PrintErrorMessage( const char *message )
{
	printf( "Erreur: %s\n", message );
}


void PrintErrorMessage( ERROR_ID error_id, const char *filename, const char *type )
{
	switch( error_id )
	{
	case ERROR_MEMORY:
		printf( "Erreur: pas assez de memoire\n" );
		break;
	case ERROR_UNABLE_OPEN_FILE:
		printf( "Erreur: impossible d'ouvrir le fichier '%s'\n", filename );
		break;
	case ERROR_UNABLE_CREATE_FILE:
		printf( "Erreur: impossible de creer le fichier '%s'\n", filename );
		break;
		case ERROR_UNABLE_SAVE_FILE:
		printf( "Erreur: impossible de sauvegarder l'image %s dans le fichier '%s'\n", type, filename );
		break;
	case ERROR_WRONG_TYPE:
		printf( "Erreur: le fichier '%s' n'est pas de type %s\n", filename, type );
		break;
	case ERROR_UNKNOWN_TYPE:
		printf( "Erreur: le fichier '%s' a un type %s inconnu\n", filename, type );
		break;
	case ERROR_WRONG_DIMENSIONS:
		printf( "Erreur: L'image '%s' n'a pas des dimensions correctes\n", filename );
		break;
	case ERROR_WRONG_DEPTH:
		printf( "Erreur: L'image '%s' n'a pas une profondeur correcte\n", filename );
		break;
	case ERROR_WRONG_CHAN_NUMBER:
		printf( "Erreur: impossible d'enregistrer l'image en %s (nombre de canaux incorrect)\n", type );
		break;
	case ERROR_CORRUPTED_FILE:
		printf( "Erreur: le fichier '%s' est corrompu\n", filename );
		break;
	}
}




// Gestion des fichiers
FileBuffer::FileBuffer( const char *filename ) : m_data(NULL), m_length(0)
{
	if( filename )
		ReadFile( filename );
}
FileBuffer::FileBuffer( const FileBuffer &fb )
{
	m_length = fb.m_length;

	if( fb.m_data )
	{
		m_data = new unsigned char[ m_length ];

		if( !m_data )
		{
			PrintErrorMessage( ERROR_MEMORY );
			return;
		}

		memcpy( m_data, fb.m_data, m_length * sizeof(unsigned char) );
	}
	else
	{
		m_data = NULL;
	}
}
FileBuffer::FileBuffer( unsigned char *data, long length ) : m_data(data), m_length(length)
{
}


FileBuffer::~FileBuffer( void )
{
	Clear();
}


bool FileBuffer::ReadFile( const char *filename )
{
	FILE *file = fopen( filename, "rb" );

	if( !file )
	{
		PrintErrorMessage( ERROR_UNABLE_OPEN_FILE, filename );
		return false;
	}

	// Calcul de sa taille
	fseek( file, 0, SEEK_END );
	long length = ftell( file );
	rewind( file );

	// On met le contenu en RAM
	unsigned char *data = new unsigned char[ length+1 ];
	if( !data )
	{
		PrintErrorMessage( ERROR_MEMORY );
		fclose( file );
		return false;
	}

	length = fread( data, sizeof(unsigned char), length, file );
	data[ length ] = 0;

	fclose( file );

	Clear();

	m_data   = data;
	m_length = length;

	return true;
}


bool FileBuffer::WriteFile( const char *filename )
{
	if( !filename || !IsValid() )
		return false;

	FILE *file = fopen( filename, "wb" );

	if( !file )
	{
		PrintErrorMessage( ERROR_UNABLE_CREATE_FILE, filename );
		return false;
	}

	long written_size = fwrite( m_data, sizeof(unsigned char), m_length, file );
	

	fclose( file );

	return (written_size == m_length);
}


void FileBuffer::Clear( void )
{
	if( m_data )
		delete [] m_data;

	m_data   = NULL;
	m_length = 0;
}
