#pragma once

#include <ImageLib.h>
#include <complex>
#include <SDL/SDL_thread.h>

typedef std::complex<double> Complex;

struct ThreadParams
{
	Complex** X;
	Complex* Exp;
	int sign;

	unsigned int x;
	unsigned int y;
	unsigned int size;
};

class FourierMT
{
public:
	FourierMT( const Image& image );
	~FourierMT();

	void FastTransform();
	Image FastTransformInverse( void );

	Image GetResult();

	void IdealFilter( unsigned int radius );
	void GaussianFilter( double sigma );

	void Save(); //Pour l'interface
	void Load(); //Pour l'interface

	static int MTTransform( void* data );
	static void Butterfly( const unsigned int l, const unsigned int c, const unsigned int size, const int sign, Complex** X, Complex* Exp, const unsigned int image_size );

private:
	Complex** m_X, *m_Exp, **m_XTemp;
	Complex** m_XSave;
	unsigned int m_C;
	unsigned int m_L;
	Image m_image;

	ThreadParams* m_threads_params;
	SDL_Thread**  m_threads;

	static unsigned int NbBits( unsigned long v );
	static void InverseBits( unsigned int* value, unsigned int nb_bits );
	void Center();
	void Reorganisation();
	void ComputeExp();
};
