#include "gradient_utils.h"




// Calcul du gradient
void Gradient::ComputeFromImage( const Image* image, GradientMaskType mask_type )
{
	assert( image->IsValid() && image->GetChanNumber() == 1 );

	Clear();
	
	const uint width        = image->GetWidth();
	const uint height       = image->GetHeight();
	const uint pixel_number = width * height;

	m_result.Create( width, height, 1 );


	// Allocations
	m_gradients   = new float*[ mask_orientation_number ];
	m_module      = new float [ pixel_number ];

	m_orientation = new GradientOrientation[ pixel_number ];

	for( uint mask_id = 0; mask_id < mask_orientation_number; mask_id++ )
	{
		m_gradients[ mask_id ] = new float[ pixel_number ];
	}

	for( uint i = 0; i < pixel_number; i++ )
	{
		for( uint mask_id = 0; mask_id < mask_orientation_number; mask_id++ )
		{
			m_gradients[ mask_id ][ i ] = 0.0f;
		}

		m_module[i]      = 0.0f;
		m_orientation[i] = orientation_unknown;
	}


	// Selection du mmasque de convolution
	const GradientMask* masks = GetMasks( mask_type );

	_DEBUG_SHOW_TIMES_START;

	// Parcours de l'image
	float module_max = 0.0f;

	for( uint y = MASK_N; y < height-MASK_N; y++ )
	{
		for( uint x = MASK_N; x < width-MASK_N; x++ )
		{
			const uint index = x + (width*y);

			// Convolution
			for( uint mask_y = 0; mask_y < MASK_SIZE; mask_y++ )
			{
				for( uint mask_x = 0; mask_x < MASK_SIZE; mask_x++ )
				{
					const uint index_pixel = (x + mask_x - MASK_N) + (width*(y + mask_y - MASK_N));

					const float gray_value = image->GetPixel( index_pixel )[ 0 ];

					for( uint mask_id = 0; mask_id < mask_orientation_number; mask_id++ )
					{
						m_gradients[ mask_id ][ index ] += gray_value * (*masks)[ mask_id ][ mask_y ][ mask_x ];
					}
				}
			}

			const float gradient_abs_e  = Abs( m_gradients[ mask_orientation_east      ][ index ] );
			const float gradient_abs_n  = Abs( m_gradients[ mask_orientation_north     ][ index ] );
			const float gradient_abs_ne = Abs( m_gradients[ mask_orientation_northeast ][ index ] );
			const float gradient_abs_nw = Abs( m_gradients[ mask_orientation_northwest ][ index ] );

			// Calcul du module

			// On recupere le gradient maximum
			m_module[ index ] = Max( gradient_abs_e   , gradient_abs_n  );
			m_module[ index ] = Max( m_module[ index ], gradient_abs_ne );
			m_module[ index ] = Max( m_module[ index ], gradient_abs_nw );

			const float module = m_module[ index ];

			if( module_max < module )
				module_max = module;


			// Calcul de l'orientation
			if( module == 0.0f )
			{
				m_orientation[ index ] = orientation_unknown;
			}
			else if( module == gradient_abs_n )
			{
				m_orientation[ index ] = orientation_north;
			}
			else if( module == gradient_abs_e )
			{
				m_orientation[ index ] = orientation_east;
			}
			else if( module == gradient_abs_ne )
			{
				m_orientation[ index ] = orientation_northeast;
			}
			else
			{
				m_orientation[ index ] = orientation_northwest;
			}
		}
	}

	_DEBUG_SHOW_TIMES_STOP( "gradients, module, orientation" );

	// Normalisation du module du gradient
	module_max = 1.0f / module_max;
	for( uint i = 0; i < pixel_number; i++ )
	{
		m_module[i] *= module_max;

		m_result.GetPixel( i )[ 0 ] = Clamp( m_module[i] * 255.0f, 0, 255 );
	}

	_DEBUG_SHOW_TIMES_STOP( "normalisation du module" );

#ifdef _DEBUG_SAVE_GRADIENTS
	for( uint mask_id = 0; mask_id < mask_orientation_number; mask_id++ )
	{
		char filename_temp[255];
		sprintf_s( filename_temp, 255, _DEBUG_SAVE_PREFIX "gradient%u.png", mask_id );

		SaveToImage( filename_temp, m_gradients[mask_id], width, height );
	}
#endif

#ifdef _DEBUG_SAVE_MODULE
	m_result.SavePNG( _DEBUG_SAVE_PREFIX "module.png" );
#endif

#ifdef _DEBUG_SAVE_ORIENTATION
	SaveToImage( _DEBUG_SAVE_PREFIX "orientation.png", reinterpret_cast< float* >(m_orientation), width, height );
#endif
}
