#include "gradient_utils.h"




// Chainage
void Gradient::Link( void )
{
	struct Chunk
	{
		uint* indices;
		uint  length;
	};

	const uint width        = m_result.GetWidth();
	const uint height       = m_result.GetHeight();
	const uint pixel_number = width * height;

	Chunk** visited = new Chunk*[ pixel_number ];
	memset( visited, 0, pixel_number*sizeof(*visited) );

	Chunk* chunks     = new Chunk[ pixel_number ];
	uint   chunks_top = 0;

	uint* stack     = new uint[ pixel_number ];
	uint  stack_top = 0;

	uint* stack_temp     = new uint[ pixel_number ];
	uint  stack_temp_top = 0;

	_DEBUG_SHOW_TIMES_START;

	// Creation des chaines
	for( uint y = 0; y < height; y++ )
	{
		for( uint x = 0; x < width; x++ )
		{
			const int index = x + width*y;

			// On a trouv un morceau de chaine
			if( m_result.GetPixel( index )[ 0 ] && ( ! visited[ index ] ))
			{
				visited[ index ] = &( chunks[ chunks_top ] );

				stack_top = 0;
				stack[ stack_top++ ] = index;

				stack_temp_top = 0;
				stack_temp[ stack_temp_top++ ] = index;

				const GradientOrientation orientation = m_orientation[ index ];

				bool bound_found = false;

				// On parcours la chaine
				while( stack_temp_top )
				{
					const int index_current = stack_temp[ --stack_temp_top ];

					const int  x_current = index_current % width;
					const int  y_current = index_current / width;

					uint neighbor_count = 0;

#define MY_CONDITION_TEMP( i ) (( ! visited[ i ] ) && m_orientation[ i ] == orientation && m_result.GetPixel( i )[ 0 ])
#define MY_CONDITION_BODY( i ) { visited[ i ] = visited[ index ]; stack[ stack_top++ ] = i; stack_temp[ stack_temp_top++ ] = i; neighbor_count++; }

					const bool on_left  = ( x_current == 0          );
					const bool on_up    = ( y_current == 0          );
					const bool on_right = ( x_current == width  - 1 );
					const bool on_down  = ( y_current == height - 1 );

					// On empile les 4-connexit en premier (important)
					int index_neighbor = index_current-1;
					if( !on_left && MY_CONDITION_TEMP( index_neighbor ) )
					{
						MY_CONDITION_BODY( index_neighbor );
					}
					index_neighbor = index_current+1;
					if( !on_right && MY_CONDITION_TEMP( index_neighbor ) )
					{
						MY_CONDITION_BODY( index_neighbor );
					}
					index_neighbor = index_current-width;
					if( !on_up && MY_CONDITION_TEMP( index_neighbor ) )
					{
						MY_CONDITION_BODY( index_neighbor );
					}
					index_neighbor = index_current+width;
					if( !on_down && MY_CONDITION_TEMP( index_neighbor ) )
					{
						MY_CONDITION_BODY( index_neighbor );
					}
					// et les 8-connexit apres
					index_neighbor = index_current-1-width;
					if( !on_left && !on_up && MY_CONDITION_TEMP( index_neighbor ) )
					{
						MY_CONDITION_BODY( index_neighbor );
					}
					index_neighbor = index_current+1-width;
					if( !on_right && !on_up && MY_CONDITION_TEMP( index_neighbor ) )
					{
						MY_CONDITION_BODY( index_neighbor );
					}
					index_neighbor = index_current+1+width;
					if( !on_right && !on_down && MY_CONDITION_TEMP( index_neighbor ) )
					{
						MY_CONDITION_BODY( index_neighbor );
					}
					index_neighbor = index_current-1+width;
					if( !on_left && !on_down && MY_CONDITION_TEMP( index_neighbor ) )
					{
						MY_CONDITION_BODY( index_neighbor );
					}

#undef MY_CONDITION_BODY
#undef MY_CONDITION_TEMP

					// On a trouv une des extremit de la chaine, on la met au dbut de la pile
					if( !bound_found && neighbor_count == 0 )
					{
						bound_found = true;

						uint temp = stack[ 0 ];
						stack[ 0 ] = stack[ stack_top-1 ];
						stack[ stack_top-1 ] = temp;
					}
				}

				// On recupere la chaine
				chunks[ chunks_top ].length  = stack_top;
				chunks[ chunks_top ].indices = new uint[ stack_top ];
				memcpy( chunks[ chunks_top ].indices, stack, stack_top * sizeof(*stack) );

				chunks_top++;
			}
		}
	}

	_DEBUG_SHOW_TIMES_STOP( "chainage" );

#ifdef _DEBUG_SAVE_LINKS
	Image image( width, height, 1 );

	for( uint i = 0; i < chunks_top; i++ )
	{
		if( chunks[i].length > 1 )
		{
			for( uint j = 0; j < chunks[i].length; j++ )
				image.GetPixel(chunks[i].indices[j])[0] = 50;

			image.GetPixel( chunks[i].indices[0]                  )[ 0 ] = 255;
			image.GetPixel( chunks[i].indices[chunks[i].length-1] )[ 0 ] = 130;
		}
	}

	image.SavePNG( _DEBUG_SAVE_PREFIX "chainage.png" );
#endif

	// Fermeture des contours
	for( uint i = 0; i < chunks_top; i++ )
	{
		Chunk* const chunk = &( chunks[i] );

		if( chunk->length < 2 )
			continue;

		uint bounds[2];
		bounds[0] = chunk->indices[ 0 ];
		bounds[1] = chunk->indices[ chunk->length-1 ];

		for( uint bound_id = 0; bound_id < 2; bound_id++ )
		{
			bool link_connected = false;

			uint index = bounds[ bound_id ];

			while( !link_connected )
			{
				const int  x = index % width;
				const int  y = index / width;

				if( x == 0 || y == 0 || x == width - 1 || y == height - 1 )
					break;

				const uint left      = index - 1;
				const uint right     = index + 1;
				const uint up        = index - width;
				const uint down      = index + width;
				const uint leftup    = index - 1 - width;
				const uint leftdown  = index - 1 + width;
				const uint rightup   = index + 1 - width;
				const uint rightdown = index + 1 + width;

				// On recupere les 3 voisins interessants
				uint neighbors[ 3 ];

				switch( m_orientation[ index ] )
				{
				case orientation_north:
					{
						if( visited[ leftup ] == chunk || visited[ left ] == chunk || visited[ leftdown ] )
						{
							neighbors[ 0 ] = right;
							neighbors[ 1 ] = rightup;
							neighbors[ 2 ] = rightdown;
						}
						else
						{
							neighbors[ 0 ] = left;
							neighbors[ 1 ] = leftup;
							neighbors[ 2 ] = leftdown;
						}
					}
					break;
				case orientation_east:
					{
						if( visited[ leftup ] == chunk || visited[ up ] == chunk || visited[ rightup ] )
						{
							neighbors[ 0 ] = down;
							neighbors[ 1 ] = leftdown;
							neighbors[ 2 ] = rightdown;
						}
						else
						{
							neighbors[ 0 ] = up;
							neighbors[ 1 ] = leftup;
							neighbors[ 2 ] = rightup;
						}
					}
					break;
				case orientation_northeast:
					{
						if( visited[ leftup ] == chunk || visited[ up ] == chunk || visited[ left ] )
						{
							neighbors[ 0 ] = rightdown;
							neighbors[ 1 ] = right;
							neighbors[ 2 ] = down;
						}
						else
						{
							neighbors[ 0 ] = leftup;
							neighbors[ 1 ] = up;
							neighbors[ 2 ] = left;
						}
					}
					break;
				case orientation_northwest:
					{
						if( visited[ up ] == chunk || visited[ rightup ] == chunk || visited[ right ] )
						{
							neighbors[ 0 ] = leftdown;
							neighbors[ 1 ] = left;
							neighbors[ 2 ] = down;
						}
						else
						{
							neighbors[ 0 ] = rightup;
							neighbors[ 1 ] = up;
							neighbors[ 2 ] = right;
						}
					}
					break;
				}

				float best_module = 0.0f;
				uint  best_neighbor;
				for( uint i = 0; i < 3; i++ )
				{
					const uint neighbor = neighbors[ i ];

					if( visited[ neighbor ] )
					{
						link_connected = true;
					}
					else if( best_module < m_module[ neighbor ] )
					{
						best_module   = m_module[ neighbor ];
						best_neighbor = neighbor;
					}
				}

				if( best_module > 0.0f && !link_connected )
				{
					index = best_neighbor;
					visited[ index ] = chunk;
					m_result.GetPixel( index )[ 0 ] = 255;
				}
				else
				{
					link_connected = true;
				}
			}
		}
	}

	_DEBUG_SHOW_TIMES_STOP( "fermeture des contours" );

#ifdef _DEBUG_SAVE_LINKS
	
	m_result.SavePNG( _DEBUG_SAVE_PREFIX "fermeture.png" );
#endif

	// Liberation de la mmoire
	for( uint i = 0; i < chunks_top; i++ )
	{
		delete [] chunks[i].indices;
	}

	delete [] stack_temp;
	delete [] stack;
	delete [] chunks;
	delete [] visited;
}
