#include "gradient_utils.h"




// Seuillage global
void Gradient::ThresholdGlobal( uchar value )
{
	const uint width        = m_result.GetWidth();
	const uint height       = m_result.GetHeight();
	const uint pixel_number = width * height;

	float threshold = value / 255.0f;

	_DEBUG_SHOW_TIMES_START;

	// Calcul du seuil
	if( value == 0 )
	{
		float sum = 0.0f;

		for( uint i = 0; i < pixel_number; i++ )
		{
			sum += m_module[i];
		}
		
		threshold = sum / pixel_number;
	}

	// Seuillage
	for( uint i = 0; i < pixel_number; i++ )
	{
		if( m_module[i] >= threshold )
		{
			m_result.GetPixel( i )[ 0 ] = 255;//Clamp< uchar, float >( m_module[i] * 255.0f, 0, 255 );
		}
		else
		{
			m_result.GetPixel( i )[ 0 ] = 0;
		}
	}

	_DEBUG_SHOW_TIMES_STOP( "seuillage global" );

#ifdef _DEBUG_SAVE_THRESHOLD
	m_result.SavePNG( _DEBUG_SAVE_PREFIX "seuillage.png" );
#endif
}


// Seuillage local
void Gradient::ThresholdLocal( int window_size )
{
	const int width  = m_result.GetWidth();
	const int height = m_result.GetHeight();

	_DEBUG_SHOW_TIMES_START;

	for( int y = 0; y < height; y++ )
	{
		for( int x = 0; x < width; x++ )
		{
			const uint index = x + width*y;

			// Calcul du seuil
			uint pixels_in_window = 0;
			float sum = 0.0f;

			int x_tmp_min = Clamp( x - window_size, 0, width-1 );
			int x_tmp_max = Clamp( x + window_size, 0, width-1 );
			int y_tmp_min = Clamp( y - window_size, 0, height-1 );
			int y_tmp_max = Clamp( y + window_size, 0, height-1 );

			for( int y_tmp = y_tmp_min; y_tmp <= y_tmp_max; y_tmp++ )
			{
				for( int x_tmp = x_tmp_min; x_tmp <= x_tmp_max; x_tmp++ )
				{
					const uint index_tmp = x_tmp + width*y_tmp;

					sum += m_module[ index_tmp ];
					pixels_in_window++;
				}
			}

			float threshold = sum / static_cast< float >(pixels_in_window);

			// Seuillage
			if( m_module[ index ] >= threshold )
			{
				m_result.GetPixel( index )[ 0 ] = 255;//Clamp< uchar, float >( m_module[ index ] * 255.0f, 0, 255 );
			}
			else
			{
				m_result.GetPixel( index )[ 0 ] = 0;
			}
		}
	}

	_DEBUG_SHOW_TIMES_STOP( "seuillage local" );

#ifdef _DEBUG_SAVE_THRESHOLD
	m_result.SavePNG( _DEBUG_SAVE_PREFIX "seuillage.png" );
#endif
}


// Seuillage par hysteresis
void Gradient::ThresholdHysteresis( uchar low, uchar high )
{
	const uint width        = m_result.GetWidth();
	const uint height       = m_result.GetHeight();
	const uint pixel_number = width * height;

	uint* stack     = new uint[ pixel_number ];
	uint  stack_top = 0;

	char* visited   = new char[ pixel_number ];

	float threshold_low  = static_cast<float>( low  ) / 255.0f;
	float threshold_high = static_cast<float>( high ) / 255.0f;

	Image image_temp( width, height, 1 );

	_DEBUG_SHOW_TIMES_START;

	// Premiere etape : on recupere les valeurs > seuil haut
	for( uint i = 0; i < pixel_number; i++ )
	{
		if( m_module[i] >= threshold_high )
		{
			stack[ stack_top++ ] = i;

			visited[i] = 1;
		}
		else
		{
			visited[i] = 0;
		}
	}

	// On tend les contours aux pixels voisins > seuil bas
	while( stack_top )
	{
		const uint index = stack[ --stack_top ];

		image_temp.GetPixel(index)[0] = 255;//Clamp< uchar, float >( m_module[index] * 255.0f, 0, 255 );

		const int x = index % width;
		const int y = index / width;

		// On empile les voisins suprieurs au second seuil
		for( int y_offset = -1; y_offset < 2; y_offset++ )
		{
			for( int x_offset = -1; x_offset < 2; x_offset++ )
			{
				const uint x_neighbor = x + x_offset;
				const uint y_neighbor = y + y_offset;

				if( x_neighbor < width && y_neighbor < height )
				{
					const uint index_neighbor = x_neighbor + (y_neighbor * width);

					if( ! visited[ index_neighbor ] && m_module[ index_neighbor ] >= threshold_low )
					{
						visited[ index_neighbor ] = 1;

						stack[ stack_top++ ] = index_neighbor;
					}
				}
			}
		}
	}

	_DEBUG_SHOW_TIMES_STOP( "seuillage par hysteresis" );

	delete [] visited;
	delete [] stack;

	m_result.Copy( &image_temp );

#ifdef _DEBUG_SAVE_THRESHOLD
	m_result.SavePNG( _DEBUG_SAVE_PREFIX "seuillage.png" );
#endif
}
