#include "gradient_utils.h"




// Fonctions locales

// Retourne un pointeurs vers une liste de masques
const GradientMask* GetMasks( Gradient::GradientMaskType type )
{
	switch( type )
	{
	case Gradient::mask_type_sobel:    return &g_mask_sobel;
	case Gradient::mask_type_kirsch:   return &g_mask_kirsch;
	case Gradient::mask_type_prewitt:  return &g_mask_prewitt;
	case Gradient::mask_type_prewitt2: return &g_mask_prewitt2;

	default: assert( false );
	}

	return NULL;
}


// Sauvegarde des donnes dans une image en niveaux de gris
void SaveToImage( const char* filename, float* data, uint width, uint height )
{
	Image image( width, height, 1 );

	uint pixel_number = width*height;

	float data_min = data[0];
	float data_max = data[0];

	for( uint i = 1; i < pixel_number; i++ )
	{
		if( data_min > data[i] )
			data_min = data[i];
		if( data_max < data[i] )
			data_max = data[i];
	}

	float value_normalised;
	float data_range = data_max - data_min;
	
	if( data_min < 0.0f )
		data_min = -data_min;

	for( uint i = 0; i < pixel_number; i++ )
	{
		value_normalised = ( (data[i] + data_min) * 255.0f ) / data_range;

		image.GetPixel(i)[0] = Clamp< uchar, float >( value_normalised, 0, 255 );
	}

	image.SavePNG( filename );
}
