#include "utils.h"

#include <math.h>




// Converti une image en niveaux de gris
void ConvertToGrayscale( Image* image )
{
	if( image->GetChanNumber() == 1 )
	{
		return;
	}

	Image image_temp( image->GetWidth(), image->GetHeight(), 1 );

	const uint pixel_number = image->GetWidth() * image->GetHeight();

	uchar* pixel_gray;
	uchar* pixel_color;
	float  grayscale;

	if( image->GetChanNumber() == 2 )
	{
		for( uint i = 0; i < pixel_number; i++ )
		{
			pixel_color = image    ->GetPixel( i );
			pixel_gray  = image_temp.GetPixel( i );

			pixel_gray[0] = pixel_color[0];
		}
	}
	else
	{
		for( uint i = 0; i < pixel_number; i++ )
		{
			pixel_color = image    ->GetPixel( i );
			pixel_gray  = image_temp.GetPixel( i );

			grayscale = ( 0.299f * pixel_color[0] ) + ( 0.587f * pixel_color[1] ) + ( 0.112f * pixel_color[2] );
				
			pixel_gray[0] = Clamp< uchar, float >( grayscale, 0, 255 );
		}
	}

	image->Copy( &image_temp );
}


// Cree l'histogramme d'une image
void ComputeHistogram( Image* histogram, const Image* source )
{
	const uint pixel_number = source->GetWidth() * source->GetHeight();

	histogram->Create( 256, 256, 1 );

	uint histo[ 256 ] = { 0 };
	uint histo_max = 0;

	for( uint i = 0; i < pixel_number; i++ )
	{
		const uchar index = source->GetPixel( i )[ 0 ];

		histo[ index ]++;

		if( histo_max < histo[ index ] )
			histo_max = histo[ index ];
	}

	for( uint i = 0; i < 256; i++ )
	{
		histo[ i ] *= 256;
		histo[ i ] = (uint)ceil( (double)histo[i] / histo_max );

		for( uint j = 256-histo[i]; j < 256; j++ )
		{
			histogram->GetPixel( i, j )[ 0 ] = 200;
		}
	}
}

#include <stdio.h>
// Seuil une image
void ThresholdGlobal( Image* image, uchar threshold, bool binarise )
{
	const uint pixel_number = image->GetWidth() * image->GetHeight();

	printf( "seuillage global " );

	if( threshold  == 0 )
	{
		// Methode automatique
		uint sum = 0;

		for( uint i = 0; i < pixel_number; i++ )
		{
			sum += image->GetPixel(i)[0];
		}

		threshold = sum / pixel_number;
		printf( "automatique " );
	}

	printf( "(%d)\n", threshold );

	for( uint i = 0; i < pixel_number; i++ )
	{
		uchar* const pixel = image->GetPixel( i );

		if( pixel[0] < threshold )
		{
			pixel[ 0 ] = 0;
		}
		else if( binarise )
		{
			pixel[ 0 ] = 255;
		}
	}
}


void ThresholdLocal( Image* image, int size, bool binarise )
{
	const int  width        = image->GetWidth();
	const int  height       = image->GetHeight();
	const uint pixel_number = width * height;

	Image result( width, height, 1 );

	for( int y = 0; y < height; y++ )
	{
		for( int x = 0; x < width; x++ )
		{
			const int x_min = Clamp< int, int >( x - size  , 0, width  );
			const int x_max = Clamp< int, int >( x + size+1, 0, width  );
			const int y_min = Clamp< int, int >( y - size  , 0, height );
			const int y_max = Clamp< int, int >( y + size+1, 0, height );

			uint sum = 0;
			uint n   = 0;
			for( int y_temp = y_min; y_temp < y_max; y_temp++ )
			{
				for( int x_temp = x_min; x_temp < x_max; x_temp++ )
				{
					sum += image->GetPixel( x_temp, y_temp )[ 0 ];
					n++;
				}
			}
			const uint threshold = sum / n;

			const uint index = x + width*y;

			uchar* const pixel        = result.GetPixel( index );
			const uchar  pixel_source = image->GetPixel( index )[ 0 ];

			if( pixel_source >= threshold )
			{
				if( binarise )
				{
					pixel[0] = 255;
				}
				else
				{
					pixel[0] = pixel_source;
				}
			}
		}
	}

	image->Copy( &result );
}


void ThresholdHysteresis( Image* image, uchar threshold_low, uchar threshold_high, bool binarise )
{
	const int width  = image->GetWidth();
	const int height = image->GetHeight();
	const uint pixel_number = width * height;

	uint* stack = new uint[ pixel_number ];
	uint  stack_top = 0;

	char* visited = new char[ pixel_number ];

	for( uint i = 0; i < pixel_number; i++ )
	{
		if( image->GetPixel(i)[0] >= threshold_high )
		{
			stack[ stack_top++ ] = i;

			visited[i] = 1;
		}
		else
		{
			visited[i] = 0;
		}
	}

	Image result( width, height, 1 );


	while( stack_top )
	{
		uint i = stack[ --stack_top ];

		if( binarise )
		{
			result.GetPixel(i)[0] = 255;
		}
		else
		{
			result.GetPixel(i)[0] = image->GetPixel(i)[0];
		}

		const int x = i % width;
		const int y = i / width;

		// On empile les voisins suprieurs au second seuil
		for( int y_offset = -1; y_offset < 2; y_offset++ )
		{
			for( int x_offset = -1; x_offset < 2; x_offset++ )
			{
				const int x_neighbor = x + x_offset;
				const int y_neighbor = y + y_offset;

				if( x_neighbor >= 0 && x_neighbor < width && y_neighbor >= 0 && y_neighbor < height )
				{
					const uint index_neighbor = x_neighbor + (y_neighbor * width);

					if( ! visited[ index_neighbor ] && image->GetPixel( index_neighbor )[0] >= threshold_low )
					{
						visited[ index_neighbor ] = 1;

						stack[ stack_top++ ] = index_neighbor;
					}
				}
			}
		}
	}

	delete [] visited;
	delete [] stack;

	image->Copy( &result );
}


// Dessine une ligne dans une image avec l'algorithme de Bresenham
void DrawLine( Image* image, int x0, int y0, int x1, int y1, uchar gray_value )
{
	bool steep = Abs( y1 - y0 ) > Abs( x1 - x0 );

	if( steep )
	{
		int tmp = x0;
		x0 = y0;
		y0 = tmp;
		tmp = x1;
		x1 = y1;
		y1 = tmp;
	}
	if( x0 > x1 )
	{
		int tmp = x0;
		x0 = x1;
		x1 = tmp;
		tmp = y0;
		y0 = y1;
		y1 = tmp;
	}

	int deltax = x1 - x0;
	int deltay = Abs( y1 - y0 );
	int error = -(deltax + 1) / 2;
	int ystep;
	int y = y0;

	ystep = ( y0 < y1 ? 1 : -1 );

	for( int x = x0; x <= x1; x++ )
	{
		if( steep )
			image->GetPixel( y, x )[0] = gray_value;
		else
			image->GetPixel( x, y )[0] = gray_value;

		error = error + deltay;

		if( error >= 0 )
		{
			y = y + ystep;
			error = error - deltax;
		}
	}
}
