#include "hough_utils.h"




// Sauvegarde des donnes dans une image en niveaux de gris
void SaveToImage( Image* image, int* data, int width, int height )
{
	image->Create( width, height, 1 );

	int pixel_number = width*height;

	int data_min = data[0];
	int data_max = data[0];

	for( int i = 1; i < pixel_number; i++ )
	{
		if( data_min > data[i] )
			data_min = data[i];
		if( data_max < data[i] )
			data_max = data[i];
	}

	int value_normalised;
	int data_range = data_max - data_min + 1;
	
	if( data_min < 0 )
		data_min = -data_min;

	for( int i = 0; i < pixel_number; i++ )
	{
		value_normalised = ( (data[i] + data_min) * 255 ) / data_range;

		image->GetPixel(i)[0] = Clamp< unsigned char, int >( value_normalised, 0, 255 );
	}
}


// Construit une image avec 3 canaux
void PrepareDestination( Image* dest, const Image* src )
{
	const int width        = src->GetWidth();
	const int height       = src->GetHeight();
	const int pixel_number = width*height;

	if( src->GetChanNumber() > 2 )
	{
		dest->Copy( src );
	}
	else
	{
		dest->Create( width, height, 3 );

		for( int i = 0; i < pixel_number; i++ )
		{
			unsigned char* p = dest->GetPixel(i);
			p[0] = src->GetPixel(i)[0];
			p[1] = p[0];
			p[2] = p[0];
		}
	}
}
