#include "fastspecification.h"

#include <string.h>

void computeCorrelation( unsigned int *f,const unsigned int* Cx, const float CyNorm[256], const float Cx_inv )
{
	unsigned int i = 0, j = 0;
	unsigned int CxCumul = Cx[0];

	while( i < 255 && j < 256 )
	{
		while( i < 255 && CxCumul * Cx_inv < CyNorm[j] )
		{
			f[i] = j;
			i++;
			CxCumul += Cx[i];
		}
		j++;
	}
	while( i < 256 ) f[i++] = j-1;
}

Image fast_specification_locale( const Image& image, unsigned int size, const Histo& histo )
{
	unsigned int chan_number = image.GetChanNumber();
	unsigned char* ptr_im;
	unsigned char* ptr_im2;
	unsigned int k;
	unsigned int i, j, in, jn, iI, jI;
	
	unsigned int** f = new unsigned int*[chan_number];
	unsigned int** h = new unsigned int*[chan_number];
	for( k = 0 ; k < chan_number ; ++k )	
	{
		f[k] = new unsigned int[ 256 ];
		h[k] = new unsigned int[ 256 ];
	}
	
	float Cx_inv = 1.0f / (float)( (2*size + 1)*( 2*size + 1) );

	Image image_result( image.GetWidth(), image.GetHeight(), chan_number );

	float histoNorm[256];
	float histo_inv = 1.0f  / (float)histo.pixelNumber();
	for( i = 0 ; i < 256 ; ++i )
	{
		histoNorm[i] = histo[i] * histo_inv;
	}

	

	//***************
	//Premiere ligne
	//***************
	i = size;
	for( k = 0 ; k < chan_number ; ++k )
	{
		memset( h[k], 0, 256 * sizeof( unsigned int ) );
	}
	// Premiere case
	for( iI = 0 ; iI < 2*size + 1 ; iI++ )
	{
		for( jI = 0 ; jI < 2*size + 1 ; jI++ )
		{
			ptr_im = image.GetPixel( jI, iI );
			for( k = 0 ; k < chan_number ; ++k )
			{
				h[k][ptr_im[k]]++;
			}
		}
	}

	ptr_im = image.GetPixel( size, i );
	ptr_im2 = image_result.GetPixel( size, i );
	for( k = 0 ; k < chan_number ; ++k )
	{
		computeCorrelation( f[k], h[k], histoNorm, Cx_inv );
		ptr_im2[k] = (unsigned char)f[k][ ptr_im[k] ];
	}

	//Coin haut gauche
	for( iI = 0 ; iI < size+1 ; iI++ )
	{
		for( jI = 0 ; jI < size+1 ;  jI++ )
		{
			ptr_im = image.GetPixel( jI, iI );
			ptr_im2 = image_result.GetPixel( jI, iI );
			for( k = 0 ; k < chan_number ; ++k )
			{
				ptr_im2[k] = (unsigned char)f[k][ ptr_im[k] ];
			}
		}
	}
	for( j = size+1 ; j < image.GetWidth() - size ; ++j )
	{
		for( iI = i - size ; iI < i + size ; iI++ )
		{
			ptr_im = image.GetPixel( j-size, iI );
			for( k = 0 ; k < chan_number ; ++k )
			{
				h[k][ptr_im[k]]--;
			}
			ptr_im = image.GetPixel( j+size, iI );
			for( k = 0 ; k < chan_number ; ++k )
			{
				h[k][ptr_im[k]]++;
			}
		}
		
		ptr_im = image.GetPixel( j, i );
		ptr_im2 = image_result.GetPixel( j, i );
		for( k = 0 ; k < chan_number ; ++k )
		{
			computeCorrelation( f[k], h[k], histoNorm, Cx_inv );
			ptr_im2[k] = (unsigned char)f[k][ ptr_im[k] ];
		}

		for( iI = 0 ; iI < size ; iI++ )
		{
			ptr_im = image.GetPixel( j, iI );
			ptr_im2 = image_result.GetPixel( j, iI );
			for( k = 0 ; k < chan_number ; ++k )
			{
				ptr_im2[k] = (unsigned char)f[k][ ptr_im[k] ];
			}
		}
	}

	//Coin haut droit
	for( iI = 0 ; iI < size+1 ; iI++ )
	{
		for( jI = image.GetWidth() - size ; jI < image.GetWidth() ;  jI++ )
		{
			ptr_im = image.GetPixel( jI, iI );
			ptr_im2 = image_result.GetPixel( jI, iI );
			for( k = 0 ; k < chan_number ; ++k )
			{
				ptr_im2[k] = (unsigned char)f[k][ ptr_im[k] ];
			}
		}
	}



	//***************
	//Autres ligne
	//***************
	for( i = size+1 ; i < image.GetHeight() - size ; ++i )
	{
		for( k = 0 ; k < chan_number ; ++k )
		{
			memset( h[k], 0, 256 * sizeof( unsigned int ) );
		}

		// Premiere case de la ligne
		for( in = 0 , iI = i - size ; in < 2*size + 1 ; in++, iI++ )
		{
			for( jn = 0 , jI = 0 ; jn < 2*size + 1 ; jn++, jI++ )
			{
				ptr_im = image.GetPixel( jI, iI );
				for( k = 0 ; k < chan_number ; ++k )
				{
					h[k][ptr_im[k]]++;
				}
			}
		}
	
		ptr_im = image.GetPixel( size, i );
		ptr_im2 = image_result.GetPixel( size, i );
		for( k = 0 ; k < chan_number ; ++k )
		{
			computeCorrelation( f[k], h[k], histoNorm, Cx_inv );
			ptr_im2[k] = (unsigned char)f[k][ ptr_im[k] ];
		}

		//Bord gauche
		for( jI = 0 ; jI < size ;  jI++ )
		{
			ptr_im = image.GetPixel( jI, i );
			ptr_im2 = image_result.GetPixel( jI, i );
			for( k = 0 ; k < chan_number ; ++k )
			{
				ptr_im2[k] = (unsigned char)f[k][ ptr_im[k] ];
			}
		}


		// Parcours ligne
		for( j = size+1 ; j < image.GetWidth() - size - 1 ; ++j )
		{
			for( iI = i - size ; iI < i + size + 1 ; iI++ )
			{
				ptr_im = image.GetPixel( j-size, iI );
				for( k = 0 ; k < chan_number ; ++k )
				{
					h[k][ptr_im[k]]--;
				}
				ptr_im = image.GetPixel( j+size, iI );
				for( k = 0 ; k < chan_number ; ++k )
				{
					h[k][ptr_im[k]]++;
				}
			}
			
			ptr_im = image.GetPixel( j, i );
			ptr_im2 = image_result.GetPixel( j, i );

			//Compute one correlation
			unsigned int a, b;
			unsigned int CxCumul;
			for( k = 0 ; k < chan_number ; ++k )
			{
				a = b = 0;
				CxCumul = h[k][0];
				while( a <= ptr_im[k] && b < 256 )
				{
					while( a <= ptr_im[k] && CxCumul * Cx_inv < histoNorm[b] )
					{
						a++;
						CxCumul += h[k][a];
					}
					b++;
				}
				ptr_im2[k] = b - 1;
			}
		}

		ptr_im = image.GetPixel( image.GetWidth() - size - 1, i );
		ptr_im2 = image_result.GetPixel( image.GetWidth() - size - 1, i );
		for( k = 0 ; k < chan_number ; ++k )
		{
			computeCorrelation( f[k], h[k], histoNorm, Cx_inv );
			ptr_im2[k] = (unsigned char)f[k][ ptr_im[k] ];
		}

		//Bord droit
		for( jI = image.GetWidth() - size ; jI < image.GetWidth() ;  jI++ )
		{
			ptr_im = image.GetPixel( jI, i );
			ptr_im2 = image_result.GetPixel( jI, i );
			for( k = 0 ; k < chan_number ; ++k )
			{
				ptr_im2[k] = (unsigned char)f[k][ ptr_im[k] ];
			}
		}
	}


	//***************
	//Derniere ligne
	//***************
	i = image.GetHeight() - size;
	for( k = 0 ; k < chan_number ; ++k )
	{
		memset( h[k], 0, 256 * sizeof( unsigned int ) );
	}
	// Premiere case de la derniere ligne
	for( iI = i - size ; iI < image.GetHeight() ; iI++ )
	{
		for( jI = 0 ; jI < 2*size + 1 ; jI++ )
		{
			ptr_im = image.GetPixel( jI, iI );
			for( k = 0 ; k < chan_number ; ++k )
			{
				h[k][ptr_im[k]]++;
			}
		}
	}

	ptr_im = image.GetPixel( size, i );
	ptr_im2 = image_result.GetPixel( size, i );
	for( k = 0 ; k < chan_number ; ++k )
	{
		computeCorrelation( f[k], h[k], histoNorm, Cx_inv );
		ptr_im2[k] = (unsigned char)f[k][ ptr_im[k] ];
	}

	//Coin bas gauche
	for( iI = i ; iI < image.GetHeight() ; iI++ )
	{
		for( jI = 0 ; jI < size+1 ;  jI++ )
		{
			ptr_im = image.GetPixel( jI, iI );
			ptr_im2 = image_result.GetPixel( jI, iI );
			for( k = 0 ; k < chan_number ; ++k )
			{
				ptr_im2[k] = (unsigned char)f[k][ ptr_im[k] ];
			}
		}
	}
	for( j = size+1 ; j < image.GetWidth() - size ; ++j )
	{
		for( iI = i - size ; iI < i + size ; iI++ )
		{
			ptr_im = image.GetPixel( j-size, iI );
			for( k = 0 ; k < chan_number ; ++k )
			{
				h[k][ptr_im[k]]--;
			}
			ptr_im = image.GetPixel( j+size, iI );
			for( k = 0 ; k < chan_number ; ++k )
			{
				h[k][ptr_im[k]]++;
			}
		}
		
		ptr_im = image.GetPixel( j, i );
		ptr_im2 = image_result.GetPixel( j, i );
		for( k = 0 ; k < chan_number ; ++k )
		{
			computeCorrelation( f[k], h[k], histoNorm, Cx_inv );
			ptr_im2[k] = (unsigned char)f[k][ ptr_im[k] ];
		}

		for( iI = i+1 ; iI < image.GetHeight() ; iI++ )
		{
			ptr_im = image.GetPixel( j, iI );
			ptr_im2 = image_result.GetPixel( j, iI );
			for( k = 0 ; k < chan_number ; ++k )
			{
				ptr_im2[k] = (unsigned char)f[k][ ptr_im[k] ];
			}
		}
	}

	//Coin bas droit
	for( iI = i ; iI < image.GetHeight() ; iI++ )
	{
		for( jI = image.GetWidth() - size ; jI < image.GetWidth() ;  jI++ )
		{
			ptr_im = image.GetPixel( jI, iI );
			ptr_im2 = image_result.GetPixel( jI, iI );
			for( k = 0 ; k < chan_number ; ++k )
			{
				ptr_im2[k] = (unsigned char)f[k][ ptr_im[k] ];
			}
		}
	}
	
	for( k = 0 ; k < chan_number ; ++k )	
	{
		delete[] f[k];
		delete[] h[k];
	}
	delete[] f;
	delete[] h;
	
	return image_result;
}



/* Cette version gere des histos sur les bords aussi -> assombrissement sur les bords */
/*Image fast_specification_locale( const Image& image, unsigned int size, const Histo& histo )
{
	unsigned int chan_number = image.GetChanNumber();
	unsigned char* ptr_im;
	unsigned char* ptr_im2;
	unsigned int k;
	int i, j, in, jn, iI, jI;
	
	unsigned int** h = new unsigned int*[chan_number];
	for( k = 0 ; k < chan_number ; ++k )	
	{
		h[k] = new unsigned int[ 256 ];
	}

	float Cx_inv = 1.0f / (float)( (2*size + 1)*( 2*size + 1) );
	
	Image image_result( image.GetWidth(), image.GetHeight(), chan_number );

	float histoNorm[256];
	float histo_inv = 1.0f  / (float)histo.pixelNumber();
	for( i = 0 ; i < 256 ; ++i )
	{
		histoNorm[i] = histo[i] * histo_inv;
	}

	
	for( i = 0 ; i < (int)image.GetHeight() ; ++i )
	{
		for( k = 0 ; k < chan_number ; ++k )
		{
			memset( h[k], 0, 256 * sizeof( unsigned int ) );
		}

		// Premiere case de la ligne
		for( in = 0 , iI = i - (int)size ; in < (int)(2*size + 1) ; in++, iI++ )
		{
			for( jn = 0 , jI = 0 ; jn < (int)size ; jn++, jI++ )
			{
				if( iI >= 0 && iI < (int)image.GetHeight() )
				{
					ptr_im = image.GetPixel( jI, iI );
					for( k = 0 ; k < chan_number ; ++k )
					{
						h[k][ptr_im[k]]++;
					}
				}
			}
		}

		// Parcours ligne
		for( j = 0 ; j < (int)image.GetWidth() ; ++j )
		{
			for( iI = i - (int)size ; iI <= i + (int)size ; iI++ )
			{
				if( iI >= 0 && iI < (int)image.GetHeight() )
				{
					if( j-(int)size >= 0 )
					{
						ptr_im = image.GetPixel( j-(int)size, iI );
						for( k = 0 ; k < chan_number ; ++k )
						{
							h[k][ptr_im[k]]--;
						}
					}
					if( j+(int)size < (int)image.GetWidth() )
					{
						ptr_im = image.GetPixel( j+(int)size, iI );
						for( k = 0 ; k < chan_number ; ++k )
						{
							h[k][ptr_im[k]]++;
						}
					}
				}
			}
			
			ptr_im = image.GetPixel( j, i );
			ptr_im2 = image_result.GetPixel( j, i );

			//Compute one correlation
			unsigned int a, b;
			unsigned int CxCumul;
			for( k = 0 ; k < chan_number ; ++k )
			{
				a = b = 0;
				CxCumul = h[k][0];
				while( a <= ptr_im[k] && b < 256 )
				{
					while( a <= ptr_im[k] && CxCumul * Cx_inv < histoNorm[b] )
					{
						a++;
						CxCumul += h[k][a];
					}
					b++;
				}
				ptr_im2[k] = b - 1;
			}
		}
	}

	for( k = 0 ; k < chan_number ; ++k )	
	{
		delete[] h[k];
	}
	delete[] h;
	
	return image_result;
}*/
