#include "specification.h"

void egalisation( Image& image )
{
	unsigned int i, j, k;
	unsigned char* ptr_im;
	unsigned int chan_number = image.GetChanNumber();
	int temp;
	
	Histo* histo = new Histo[ chan_number ];
	Histo* histoC = new Histo[ chan_number ];
	for( k = 0 ; k < chan_number ; ++k )
	{
		histo[k].computeFromImage( image, k );
		histoC[k] = histo[k].cumulated();
	}

	for( j = 0 ; j < image.GetHeight() ; j++ )
	{
		for( i = 0 ; i < image.GetWidth() ; i++ )
		{
			ptr_im = image.GetPixel( i, j );
			for( k = 0 ; k < chan_number ; ++k )
			{
				temp = histoC[k][ ptr_im[k] ] * 256 / histo[k].pixelNumber() ;
				if( temp > 255 ) temp = 255;
				if( temp < 0 ) temp = 0;
				ptr_im[k] = (unsigned char)temp;
			}
		}
	}
	
	delete[] histo;
	delete[] histoC;
}

Image egalisation_locale( const Image& image, unsigned int size )
{
	unsigned int chan_number = image.GetChanNumber();
	unsigned char* ptr_im;
	unsigned char* ptr_im2;
	unsigned int k;
	unsigned int i, j, in, jn, iI, jI;
	int temp;
	
	Histo* histo = new Histo[ chan_number ];
	
	Image image_result( image.GetWidth(), image.GetHeight(), chan_number );
	
	for( i = size ; i < image.GetHeight() - size ; ++i )
	{
		for( j = size ; j < image.GetWidth() - size ; ++j )
		{
			for( k = 0 ; k < chan_number ; ++k )
			{
				histo[k].clear();
			}
		
			for( in = 0 , iI = i - size ; in < 2*size + 1 ; in++, iI++ )
			{
				for( jn = 0 , jI = j - size ; jn < 2*size + 1 ; jn++, jI++ )
				{
					ptr_im = image.GetPixel( jI, iI );
					for( k = 0 ; k < chan_number ; ++k )
					{
						histo[k].addPixel( ptr_im[k] );
					}
				}
			}
			
			ptr_im = image.GetPixel( j, i );
			ptr_im2 = image_result.GetPixel( j, i );
			for( k = 0 ; k < chan_number ; ++k )
			{
				temp = histo[k].cumulated()[ ptr_im[k] ] * 256 / ( histo[k].pixelNumber() ) ;
				if( temp > 255 ) temp = 255;
				if( temp < 0 ) temp = 0;
				ptr_im2[k] = (unsigned char)temp;
			}
			
		}
	}
	
	delete[] histo;
	
	return image_result;
}


void computeCorrelation( unsigned int f[256], const Histo& Cx, const Histo& Cy )
{
	assert( Cx.pixelNumber() > 0 && Cy.pixelNumber() > 0 );
	unsigned int i = 0, j = 0;

	float Cx_inv = 1.0f / (float)(Cx.pixelNumber());
	float Cy_inv = 1.0f / (float)(Cy.pixelNumber());
	
	while( i < 255 && j < 256 )
	{
		float CyNorm = Cy[j] * Cy_inv;
		while( i < 255 && Cx[i] * Cx_inv < CyNorm )
		{
			f[i] = j;
			i++;
		}
		j++;
	}
	while( i < 256 ) f[i++] = j-1;
}

void specification( Image& image, const Histo& histo )
{
	unsigned int i, j, k;
	unsigned char* ptr_im;
	unsigned int chan_number = image.GetChanNumber();
	
	unsigned int* f[3];
	for( k = 0 ; k < chan_number ; ++k )	
	{
		f[k] = new unsigned int[ 256 ];
		for( i = 0 ; i < 256 ; ++ i )
			f[k][i] = 0;
	}
		
	
	Histo* h = new Histo[ chan_number ];
	for( k = 0 ; k < chan_number ; ++k )
	{
		h[k].computeFromImage( image, k );
		computeCorrelation( f[k], h[k].cumulated(), histo );
	}

	for( j = 0 ; j < image.GetHeight() ; j++ )
	{
		for( i = 0 ; i < image.GetWidth() ; i++ )
		{
			ptr_im = image.GetPixel( i, j );
			for( k = 0 ; k < chan_number ; ++k )
			{
				ptr_im[k] = (unsigned char)f[k][ ptr_im[k] ];
			}
		}
	}

	for( k = 0 ; k < chan_number ; ++k )	
	{
		delete[] f[k];
	}
	delete[] h;
	
}

Image specification_locale( const Image& image, unsigned int size, const Histo& histo )
{
	unsigned int chan_number = image.GetChanNumber();
	unsigned char* ptr_im;
	unsigned char* ptr_im2;
	unsigned int k, k2;
	unsigned int i, j, in, jn, iI, jI;
	
	unsigned int* f[4];
	for( k = 0 ; k < chan_number ; ++k )	
	{
		f[k] = new unsigned int[ 256 ];
	}
	
	Histo* h = new Histo[ chan_number ];
	
	Image image_result( image.GetWidth(), image.GetHeight(), chan_number );
	
	for( i = size ; i < image.GetHeight() - size ; ++i )
	{
		for( j = size ; j < image.GetWidth() - size ; ++j )
		{
			for( k = 0 ; k < chan_number ; ++k )
			{
				h[k].clear();
				for( k2 = 0 ; k2 < 256 ; ++k2 )
				{
					f[k][k2] = 0;
				}
			}
		
			for( in = 0 , iI = i - size ; in < 2*size + 1 ; in++, iI++ )
			{
				for( jn = 0 , jI = j - size ; jn < 2*size + 1 ; jn++, jI++ )
				{
					ptr_im = image.GetPixel( jI, iI );
					for( k = 0 ; k < chan_number ; ++k )
					{
						h[k].addPixel( ptr_im[k] );
					}
				}
			}
			
			ptr_im = image.GetPixel( j, i );
			ptr_im2 = image_result.GetPixel( j, i );
			for( k = 0 ; k < chan_number ; ++k )
			{
				computeCorrelation( f[k], h[k].cumulated(), histo );
				ptr_im2[k] = (unsigned char)f[k][ ptr_im[k] ];
			}
			
		}
	}
	
	for( k = 0 ; k < chan_number ; ++k )	
	{
		delete[] f[k];
	}
	delete[] h;
	
	return image_result;
}
