#include "halftoning.h"
#include <cstdlib>
#include <cstdio>
#include <cassert>

#define CLAMP( x, min, max ) { if( (x) < (min) ) (x) = (min); if( (x) > (max) ) (x) = (max); }



void halfToning( Image *dst, Image *src, MatrixType matrix_type, bool clamp )
{
	assert( src && dst && src->GetWidth() == dst->GetWidth() && src->GetHeight() == dst->GetWidth() );

	Image image_temp( *dst );

	// Conversion en niveaux de gris
	int nb_pixels = src->GetWidth() * src->GetHeight();
	for( int i = 0; i < nb_pixels; i++ )
	{
		unsigned char *pixel = src->GetPixel( i );

		if( src->GetChanNumber() > 2 )
		{
			double pixel_temp = 0.299*static_cast<double>(pixel[0]) + 0.587*static_cast<double>(pixel[1]) + 0.514*static_cast<double>(pixel[2]);

			CLAMP( pixel_temp, 0.0, 255.0 );
			image_temp.GetPixel(i)[0] = static_cast<unsigned char>(pixel_temp);
		}
		else
		{
			image_temp.GetPixel(i)[0] = pixel[0];
		}
	}

	
	int matrix_width = 0;
	int matrix_height = 0;
	double *matrix = NULL;
	double matrix_floyd[]  = { 0.0, 0.0, 7.0/16.0, 3.0/16.0, 5.0/16.0, 1.0/16.0 };
	double matrix_jarvis[] = { 0.0, 0.0, 0.0, 7.0/48.0, 5.0/48.0, 3.0/48.0, 5.0/48.0, 7.0/48.0, 5.0/48.0, 3.0/48.0, 1.0/48.0, 3.0/48.0, 5.0/48.0, 3.0/48.0, 1.0/48.0 };
	double matrix_stucki[] = { 0.0, 0.0, 0.0, 8.0/42.0, 4.0/42.0, 2.0/42.0, 4.0/42.0, 8.0/42.0, 4.0/42.0, 2.0/42.0, 1.0/42.0, 2.0/42.0, 4.0/42.0, 2.0/42.0, 1.0/42.0 };



	switch( matrix_type )
	{
	case Floyd_Steinberg:
		matrix_width = 1;
		matrix_height = 2;
		matrix = matrix_floyd;
		break;
	case Jarvis_Judis_Ninke:
		matrix_width = 2;
		matrix_height = 3;
		matrix = matrix_jarvis;
		break;
	case Stucki:
		matrix_width = 2;
		matrix_height = 3;
		matrix = matrix_stucki;
		break;

	}
	
	for( unsigned int y = 0; y < dst->GetHeight(); y++ )
	{
		for( unsigned int x = 0; x < dst->GetWidth(); x++ )
		{
			unsigned char *pixel_src = image_temp.GetPixel( x, y );


			int error_value = 0;

			if( *pixel_src > 127 )
			{
				dst->GetPixel( x, y )[0] = 255;

				error_value = *pixel_src - 255;
			}
			else
			{
				dst->GetPixel( x, y )[0] = 0;

				error_value = *pixel_src;
			}

			// Diffusion d'erreur sur image_temp
			int matrix_x = matrix_width;
			for( int matrix_y = 0; matrix_y < matrix_height; matrix_y++ )
			{
				while( matrix_x < matrix_width*2 )
				{
					unsigned int diffusion_x = x + matrix_x;
					unsigned int diffusion_y = y + matrix_y;
					int index = (matrix_x+matrix_width) + (matrix_width*2 + 1)*matrix_y;
					
					if( diffusion_x < image_temp.GetWidth()-1 && diffusion_y < image_temp.GetHeight()-1 )
					{
						unsigned char *pixel = image_temp.GetPixel( diffusion_x, diffusion_y );
						
						double diffusion_value = matrix[index] * static_cast<double>(error_value);
						if(clamp) CLAMP( diffusion_value, 0, 255 );

						int pixel_temp = static_cast<int>(*pixel) + static_cast<int>(diffusion_value);
						CLAMP( pixel_temp, 0, 255 );
						*pixel = pixel_temp;
					}

					matrix_x++;
				}

				matrix_x = -matrix_width;
			}
		}
	}
}
