#include "histo.h"

Histo::Histo( unsigned int _size )
{
	m_size = _size;
	m_tab.resize( _size, 0 );
	m_pixel_number = 0;
	m_max = 0;
}
Histo::Histo( const Histo& _histo )
{
	m_size = _histo.m_size;
	m_tab = _histo.m_tab;
	m_pixel_number = _histo.m_pixel_number;
	m_max = _histo.m_max;
}
Histo::Histo( const Image& _image, unsigned int _channel )
{
	computeFromImage( _image, _channel );
}
Histo::~Histo()
{
}


bool Histo::load( const char *_filename )
{
	FILE* file = fopen( _filename, "rb" );
	if( !file ) return false;
	
	fscanf( file, "size : %u\n", &m_size );
	
	m_tab.resize( m_size, 0 );
	m_max = 0;
	m_pixel_number = 0;
	
	for( unsigned int i = 0 ; i < m_size ; ++i )
	{
		fscanf( file, "%u\n", &m_tab[i] );
		m_pixel_number += m_tab[i];
		if( m_tab[i] > m_max ) m_max = m_tab[i];
	}
	fclose( file );
	
	return true;
}
bool Histo::save( const char *_filename ) const
{
	FILE* file = fopen( _filename, "wb" );
	if( !file ) return false;
	
	fprintf( file, "size : %u\n", m_size );
	
	for( unsigned int i = 0 ; i < m_size ; ++i )
	{
		fprintf( file, "%u\n", m_tab[i] );
	}
	fclose( file );
	
	return true;
}

Image Histo::toImage( unsigned int _height ) const
{
	Image image( 256, _height, 1 );
	for( unsigned int i = 0 ; i < _height ; ++i )
	{
		for( unsigned int j = 0 ; j < 256 ; ++j )
		{
			if( m_max - i * ( m_max / (double) _height ) < m_tab[j] )
				*image.GetPixel( j, i ) = 255;
		}
	}
	return image;
}

void Histo::computeFromImage( const Image& _image, unsigned int _channel )
{
	m_size = 256;
	m_tab.resize( m_size, 0 );
	m_pixel_number = _image.GetWidth() * _image.GetHeight();
	m_max = 0;
	
	for( unsigned int i = 0 ; i < _image.GetHeight() ; ++i )
	{
		for( unsigned int j = 0 ; j < _image.GetWidth() ; ++j )
		{
			unsigned char position = _image.GetPixel( j, i )[_channel];
			m_tab[ position ]++;
			if( m_tab[ position ] > m_max ) m_max = m_tab[ position ];
		}
	}
}
Histo Histo::cumulated()
{
	Histo histoC( *this );
	for( unsigned int i = 1 ; i < m_size ; ++i )
	{
		histoC.m_tab[i] += histoC.m_tab[i-1];
	}
	
	return histoC;
}

void Histo::clear()
{
	m_tab.clear();
	m_tab.resize( m_size, 0 );
	m_pixel_number = 0;
	m_max = 0;
}

void Histo::addPixel( const unsigned int _index )
{
	m_tab[_index]++;
	m_pixel_number++;
	if( m_tab[ _index ] > m_max ) m_max = m_tab[ _index ];
}
bool Histo::removePixel( const unsigned int _index )
{
	if( m_tab[_index] > 0 )
	{
		m_tab[_index]--;
		m_pixel_number--;
		return true;
	}
	return false;
}

unsigned int & Histo::operator[]( const unsigned int _index )
{
	assert( _index < m_size );
	return m_tab[_index];
}
const unsigned int Histo::operator[]( const unsigned int _index ) const
{
	assert( _index < m_size );
	return m_tab[_index];
}

const Histo& Histo::operator=( const Histo& _histo )
{
	assert( m_size != _histo.m_size );
	for( unsigned int i = 0 ; i < m_size ; ++i )
	{
		m_tab[i] = _histo.m_tab[i];
	}
	m_pixel_number = _histo.m_pixel_number;
	m_max = _histo.m_max;

	return *this;
}